#extension GL_ARB_texture_rectangle : enable

uniform sampler2DRect TextureParticles, TextureVelocities;

uniform vec2 p_texture_size;	// particle texture size
uniform vec3 res;
uniform vec3 res_sub_1;
uniform float ft_z_mod;
uniform float emission_row;
uniform float emission_position;


/**
 * particles are just handled in the area [0..1][0..1][0..1]
 * therefore for rendering they have to be translated by vec3(0.5)!
 */

vec3 getVector(vec3 p)
{
	p = clamp(p, vec3(0), res_sub_1);
	vec2 tmpv = vec2(mod(p.z, ft_z_mod), floor(p.z / ft_z_mod));
	return vec3(texture2DRect(TextureVelocities, p.xy + tmpv*res.xy));
}


void main(void)
{
	vec2 pr = floor(vec2(gl_TexCoord[0]));

	if (pr.y == emission_row)
	{
		// emit new particle
		gl_FragColor = vec4(	emission_position,		// the "column"
					(pr.x+0.5) / p_texture_size.x,	// the y position [0.5;texture_size.x-0.5]
					2.5/res.z,			// z position -> is going to be clamped
					1.0);				// MY one and only ONE!!!
		return;
	}

	// set new position
	vec3 particle_pos = vec3(texture2DRect(TextureParticles, pr));
	if (	particle_pos.x > 1.0 ||
		particle_pos.y > 1.0 ||
		particle_pos.z > 1.0 ||
		particle_pos.x < 0.0 ||
		particle_pos.y < 0.0 ||
		particle_pos.z < 0.0)
	{
		gl_FragColor = vec4(0.0,0.0,100.0, 1.0);		// set z to 100 (out of clipping area)
		return;
	}

// TODO: use interpolation (e.g. AUTOMATICALLY BY GPU INTERPOLATION!)
	for (int i = 0; i < 100; i++)
	{
		// use x resolution to scale the velocity!
		particle_pos += getVector(floor(particle_pos*res))*0.01/res[0];
	}
	gl_FragColor = vec4(particle_pos, 1.0);
	return;

	gl_FragColor = vec4(particle_pos + getVector(floor(particle_pos*res)), 1.0);
}
