#extension GL_ARB_texture_rectangle : enable
#extension GL_ARB_draw_buffers : enable

uniform sampler2DRect TextureDF;
uniform sampler2DRect TextureFluidMass;
uniform sampler2DRect TextureFluidFraction;
uniform sampler2DRect TextureVelocityDensity;

uniform vec3 res;
uniform vec3 res_sub_1;
uniform float ft_z_mod;

vec2 getFlatTexturePosition(float x, float y, float z)
{
	vec3 p = clamp(vec3(x,y,z), vec3(0), res_sub_1);
	vec2 tmpv = vec2(mod(p.z, ft_z_mod), floor(p.z / ft_z_mod));
	return p.xy + tmpv*res.xy;
}

float sumVector(vec4 v)
{
	vec2 a = v.rg+v.ba;
	return a.r + a.g;
}

float sumVector2(vec2 v)
{
	return v.r + v.g;
}


#include "shader/feqs.inc"
#include "shader/config.inc"
#include "shader/math.inc"

/**
 * output:
 * gl_FragData[0]: new density distributions
 * gl_FragData[1]: mass textur
 */

void main(void)
{
	vec2 pr = floor(vec2(gl_TexCoord[0]));	// rectangle coordinates

	vec3 pos;	// 3d position
	pos.xy = floor(pr.xy / res.xy);
	pos.z = pos.x + pos.y * ft_z_mod;
	pos.xy = mod(pr.xy, res.xy);

	// load the fluid fraction and fluid mass of current cell
	vec2 local_fluid_fraction = texture2DRect(TextureFluidFraction, pr).rg;

	if (local_fluid_fraction.g < -1.0)
	{
		gl_FragData[0] = vec4(0.1);
		gl_FragData[1] = vec4(0.0);
		return;
	}

	float local_fluid_mass = texture2DRect(TextureFluidMass, pr).r;
	

#if DEBUG_SHADER
	vec4 df = texture2DRect(TextureDF, pr);
	if (df.r == 666.0)
	{
//		gl_FragData[0] = df;
		gl_FragData[0] = vec4(0, 100, 0, 0);
		gl_FragData[1] = texture2DRect(TextureFluidMass, pr);
		return;
	}
#endif


	/*
	 * this cell now is wheater an obstacle, fluid or an interface cell
	 * the further handling depends on the neighbored flags
	 */
