
	vec4 new_densities = vec4(
		texture2DRect(TextureDF, p0).r,
		texture2DRect(TextureDF, p1).g,
		texture2DRect(TextureDF, p2).b,
		texture2DRect(TextureDF, p3).a
	);

#if HANDLE_GAS
	if (local_fluid_fraction.g == -1.0)	// GAS CELL
	{
		/*
		 * if we handle the free surface, we can set the DF's to randomly selected
		 * variables to check the implementation for bugs because these values have to be reconstructed!
		 */
		// TODO
		gl_FragData[0] = new_densities;
gl_FragData[0] = vec4(1000, 0, 1000, 0);
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
	}
#else
	if (local_fluid_fraction.g == -1.0)	// GAS CELL
	{
		/*
		 * no mass exchange from gas and obstacle cells
		 * gas cells dont have a mass but we store the value because we cannot
		 * distribute negative masses to neighbored nodes.
		 *
		 * we dont handle obstacle cells here because the mass is conservated
		 * by the bounce back method - so obstacle cells are just handled as fluid
		 */
		gl_FragData[0] = new_densities;
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
	}
#endif

// OBSTACLE
	if (local_fluid_fraction.g == 2.0)
	{
		gl_FragData[0] = new_densities;
		gl_FragData[1] = vec4(0, 0, 0, 0);
		return;
	}

	vec4 local_density_distributions = texture2DRect(TextureDF, pr);

	vec4 tmpv;
	vec4 neighbor_fluid_fraction;
	vec4 neighbor_fluid_flags;

	tmpv = vec4(
		texture2DRect(TextureFluidFraction, p0).rg,
		texture2DRect(TextureFluidFraction, p1).rg
	);

	neighbor_fluid_fraction.rg = tmpv.rb;
	neighbor_fluid_flags.rg = tmpv.ga;

	tmpv = vec4(
		texture2DRect(TextureFluidFraction, p2).rg,
		texture2DRect(TextureFluidFraction, p3).rg
	);

	neighbor_fluid_fraction.ba = tmpv.rb;
	neighbor_fluid_flags.ba = tmpv.ga;

	/**
	 * we setup flag vectors (0 or 1) with the corresponding states for the adjacent cells
	 * because we want to handle obstacles as fluid, we reconstruct the flagFluid
	 * by using flagInterface and flagsGas
	 * there are only 4 states: GAS, INTERFACE, FLUID, OBSTACLE
	 */

	vec4 flagsGas		= vec4(equal(neighbor_fluid_flags, vec4(-1.0)));
	vec4 flagsInterface	= vec4(equal(neighbor_fluid_flags, vec4(0.0)));
	vec4 flagsFluid		= vec4(equal(neighbor_fluid_flags, vec4(1.0)));
	vec4 flagsObstacle	= vec4(equal(neighbor_fluid_flags, vec4(2.0)));

	vec4 flagsFluidOrObstacle	= (vec4(1.0) - flagsGas)*(vec4(1.0) - flagsInterface);
	vec4 flagsFluidOrInterface	= (vec4(1.0) - flagsGas)*(vec4(1.0) - flagsObstacle);

// FLUID
	if (local_fluid_fraction.g == 1.0)
	{
		/*
		 * this cell is a normal fluid cell - sum up the mass exchange
		 * the same amount of fluid which is going into obstacle cells is also send back by the bounce back method!
		 * use flags to avoid transfer to obstacles because mass can disappear if fluid moves along obstacle border
		 *
		 * only exchange mass with fluid or interfaces
		 */
		local_fluid_mass += dot((new_densities - local_density_distributions.grab), flagsFluidOrInterface)*MASS_EXCHANGE_FACTOR;

		gl_FragData[0] = new_densities;
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
	}


#if DEBUG_SHADER
	if (local_fluid_fraction.g != 0.0)
	{
		gl_FragData[0] = vec4(100, 0, 0, 0);
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
	}
#endif

	/*
	 * interface cell - assume(local_fluid_fraction.g == 0.0)
	 * the only special case we have to care for is if the other neighbor is also an interface cell.
	 *
	 * 1st step:
	 * load the fluid fraction values to neighbor_fluid_fraction
	 * and the flags to neighbor_fluid_flags
	 *
	 *
	 * TODO: use mix function
	 */

	local_fluid_mass += dot(
		(new_densities - local_density_distributions.grab) ,
			(
			flagsFluid +
			flagsInterface * (vec4(local_fluid_fraction.r) + neighbor_fluid_fraction) * vec4(0.5)
			)
		)*MASS_EXCHANGE_FACTOR;

	/*
	 * recomputate density distributions from gas cells
	 * is not generated if propagate_gas_flag is set to false (see inc file)
	 */

	vec4 u = texture2DRect(TextureVelocityDensity, pr);

