#include "shader/propagation_header.inc"

	vec2 p0 = getFlatTexturePosition(pos.x, pos.y, pos.z-1.0);
	vec2 p1 = getFlatTexturePosition(pos.x, pos.y, pos.z+1.0);

	vec4 new_densities = vec4(
		texture2DRect(TextureDF, p0).r,
		texture2DRect(TextureDF, p1).g,
		texture2DRect(TextureDF, pr).ba
	);

// GAS
	/**
	 * we also handle the obstacles here since there's no mass exchange
	 * between fluid and obstacles
	 */
	if (local_fluid_fraction.g == -1.0)	// GAS CELL
	{
#if HANDLE_GAS
		/*
		 * if we handle the free surface, we can set the DF's to randomly selected variables to
		 * check the implementation for bugs because these values have to be reconstructed!
		 */
		gl_FragData[0] = new_densities;
gl_FragData[0] = vec4(1000, 0, 1000, new_densities.a);	// set to bogus data to test correct reconstruction
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
#else
		// no MASS EXCHANGE with gas cells
		gl_FragData[0] = new_densities;
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
#endif
	}

// OBSTACLE
	if (local_fluid_fraction.g == 2.0)	// OBSTACLES - no mass exchange
	{
		gl_FragData[0] = new_densities;
		gl_FragData[1] = vec4(0, 0, 0, 0);
		return;
	}

	vec4 local_density_distributions = texture2DRect(TextureDF, pr);
/*
// TODO
gl_FragData[0] = local_density_distributions;
gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
return;
*/
	vec4 tmpv;

	// load neighbored fluid fractions
	tmpv = vec4(
		texture2DRect(TextureFluidFraction, p0).rg,
		texture2DRect(TextureFluidFraction, p1).rg
	);

	vec2 neighbor_fluid_fraction = tmpv.rb;
	vec2 neighbor_fluid_flags = tmpv.ga;

	vec2 flagsGas		= vec2(equal(neighbor_fluid_flags, vec2(-1.0)));
	vec2 flagsInterface	= vec2(equal(neighbor_fluid_flags, vec2(0.0)));
	vec2 flagsFluid		= vec2(equal(neighbor_fluid_flags, vec2(1.0)));
	vec2 flagsObstacle	= vec2(equal(neighbor_fluid_flags, vec2(2.0)));

	vec2 flagsFluidOrObstacle	= (vec2(1.0) - flagsGas)*(vec2(1.0) - flagsInterface);
	vec2 flagsFluidOrInterface	= (vec2(1.0) - flagsGas)*(vec2(1.0) - flagsObstacle);

// FLUID
	if (local_fluid_fraction.g == 1.0)
	{
		local_fluid_mass += sum((new_densities.rg - local_density_distributions.gr)*flagsFluidOrInterface)*MASS_EXCHANGE_FACTOR;

		gl_FragData[0] = new_densities;
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
	}

#if DEBUG_SHADER
	if (local_fluid_fraction.g != 0.0)
	{
		gl_FragData[0] = vec4(100, 0, 0, 0);
		gl_FragData[1] = vec4(local_fluid_mass, 0, 0, 0);
		return;
	}
#endif

// INTERFACE

	local_fluid_mass += sum(
		(new_densities.rg - local_density_distributions.gr) *
			(
			flagsFluid +
			flagsInterface * (vec2(local_fluid_fraction.r) + neighbor_fluid_fraction) * vec2(0.5)
			)
		)*MASS_EXCHANGE_FACTOR;

	/*
	 * recomputate density distributions from gas cells
	 */
	vec4 u = texture2DRect(TextureVelocityDensity, pr);

#if HANDLE_GAS
	u[3] = 1.0;
	vec3 feq = f_eq4(u);

	vec2 bothGas = flagsGas.gr*flagsGas.rg;
	new_densities.rg =	(vec2(1.0)-flagsGas)*new_densities.rg +		// FLUID or INTERFACE (not gas | obstacle)
				flagsGas *
				(
					(vec2(1.0) - bothGas)*	(feq.rg + feq.gr - new_densities.gr) +
					bothGas*		(feq.rg)
				);
//new_densities.rg = feq.rg;
#endif

#include "shader/propagation_footer.inc"
