#extension GL_ARB_texture_rectangle : enable
#extension GL_ARB_draw_buffers : enable 

/**
 * this shander handels the gas and interface cells
 * if the fluid 
 * change gas to interface cells and interface cells to fluid cells
 */

#include "shader/config.inc"


uniform vec3 res;
uniform vec3 res_sub_1;
uniform float ft_z_mod;

uniform sampler2DRect TextureFluidFraction;	// the new fluid fractions include the fluid fractions and the obstacle data as the 2 flag

/**
 * get fraction data (fluid fraction + flag)
 */
vec2 getFrac(float x, float y, float z)
{
	vec3 p = clamp(vec3(x,y,z), vec3(0.0), res_sub_1);

	vec2 tmpv = vec2(mod(p.z, ft_z_mod), floor(p.z / ft_z_mod));
	return texture2DRect(TextureFluidFraction, p.xy + tmpv*res.xy).rg;
}


void main(void)
{
	vec2 pr = floor(vec2(gl_TexCoord[0]));	// rectangle coordinates
	vec4 fluid_fraction = texture2DRect(TextureFluidFraction, pr);

	vec3 pos;	// 3d position
	pos.xy = floor(pr.xy/res.xy);
	pos.z = pos.x + pos.y * ft_z_mod;
	pos.xy = mod(pr.xy, res.xy);

#if DEBUG_SHADER
		/**
		 * DEBUG_SHADER:
		 * check if there's a gas cell with a neighbored fluid cell and also other direction
		 * => set white color for error
		 */
		if (fluid_fraction.g == FLAGS_GAS)
		{
			// check for adjacent flag which is a gas cell
			float adjacent_check_flag = FLAGS_FLUID;
			vec4 return_frag_color = vec4(1.0, 4.0, 0.0, 0.0);
#include "shader/update_cell_state_check.frag"
		}
		else if (fluid_fraction.g == FLAGS_FLUID)
		{
			// check for adjacent flag which is a fluidcell
			float adjacent_check_flag = FLAGS_GAS;
			vec4 return_frag_color = vec4(1.0, 4.0, 0.0, 0.0);
#include "shader/update_cell_state_check.frag"
		}

#endif

/*
 * FLUID and OBSTACLE
 * dont handle fluid and obstacle cells here
 */

	if (fluid_fraction.g == FLAGS_FLUID || fluid_fraction.g == FLAGS_OBSTACLE)
	{
		gl_FragColor = fluid_fraction;
		return;
	}

/*
 * INTERFACE CELL (g == 0)
 * change to fluid cell if fluid_fraction > 1.0 + EXTRA_OFFSET
 */
	if (fluid_fraction.g == FLAGS_INTERFACE)	// interface cell?
	{
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.r > 1.0 + EXTRA_OFFSET)
		{
			// change to fluid cell
			// after this cell becomes fluid, the neighbored gas cells have to be checked
			// whether they have to be changed to interface cells
			gl_FragColor = vec4(fluid_fraction.r, FLAGS_FLUID, 0, 0);	// new fluid
			return;
		}
#else
		if (fluid_fraction.r > 1.0 + EXTRA_OFFSET)
		{
			gl_FragColor = vec4(fluid_fraction.r, FLAGS_FLUID, 0, 0);	// new fluid
			return;
		}
		else
		{
			// change nothing
			gl_FragColor = fluid_fraction;			// stay interface cell
			return;
		}
#endif
		// after this we analyze if we can delete this interface cell
		// otherwise we return
	}

#if DEBUG_SHADER
	if (fluid_fraction.g != FLAGS_GAS && fluid_fraction.g != FLAGS_INTERFACE)	// gas + interface cell
	{
		gl_FragColor = vec4(0, 4, 0, 0);
		return;
	}
#endif


	// the REMAINING cells which are handled now are GAS CELLS

	// if "REMOVE_LONELY_INTERFACE_CELLS" is active, also interface cells are handeled

/**
 * remove the interface if all the neighbors are whether fluid or gas cells
 */
	vec4 tmp, tmp2, flags, frac;

	tmp.rg = getFrac(pos.x-1.0, pos.y, pos.z);	// load first neighbors
	tmp.ba = getFrac(pos.x+1.0, pos.y, pos.z);
	tmp2.rg = getFrac(pos.x, pos.y-1.0, pos.z);
	tmp2.ba = getFrac(pos.x, pos.y+1.0, pos.z);
	flags = vec4(tmp.ga, tmp2.ga);			// flags for neighbored cells

#if REMOVE_LONELY_INTERFACE_CELLS
	bool anyGasNeighbor = false;
	bool anyFluidNeighbor = false;

	// if REMOVE_LONELY_INTERFACE_CELLS is active, we have to check whether this
	// cell is a gas flag because this is assumed
	if (fluid_fraction.g == FLAGS_GAS)
	{
#endif
		if (any(equal(flags, vec4(FLAGS_INTERFACE))))	// adjacent cell is an interface cell
		{
			frac = vec4(tmp.rb, tmp2.rb);
			if (any(greaterThan(frac, vec4(1.0 + EXTRA_OFFSET))))	// new neighbored fluid...
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_INTERFACE,0,0);			// ... then this becomes an interface (special flag -10) => reconstruct DF's
#else
				gl_FragColor = vec4(0,FLAGS_INTERFACE,0,0);	// ... then this becomes an interface => reconstruct DF's
#endif
				return;
			}
		}
#if REMOVE_LONELY_INTERFACE_CELLS
	}
	else
	{
		// check if the interface cell has any gas or fluid neighbor
		anyGasNeighbor = any(equal(flags, vec4(-1)));
		anyFluidNeighbor = any(equal(flags, vec4(+1)));
	}
#endif

	tmp.rg = getFrac(pos.x+1.0, pos.y-1.0, pos.z);
	tmp.ba = getFrac(pos.x+1.0, pos.y+1.0, pos.z);
	tmp2.rg = getFrac(pos.x-1.0, pos.y+1.0, pos.z);
	tmp2.ba = getFrac(pos.x-1.0, pos.y-1.0, pos.z);
	flags = vec4(tmp.ga, tmp2.ga);

#if REMOVE_LONELY_INTERFACE_CELLS
	if (fluid_fraction.g == -1.0)
	{
#endif
		if (any(equal(flags, vec4(FLAGS_INTERFACE))))
		{
			frac = vec4(tmp.rb, tmp2.rb);
			if (any(greaterThan(frac, vec4(1.0 + EXTRA_OFFSET))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_INTERFACE,0,0);		// ... then this becomes an interface => reconstruct DF's
#else
				gl_FragColor = vec4(0,0,0,0);		// ... then this becomes an interface => reconstruct DF's
#endif
				return;
			}
		}
#if REMOVE_LONELY_INTERFACE_CELLS
	}
	else
	{
		anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(-1)));
		anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(+1)));
	}
#endif

	tmp.rg = getFrac(pos.x+1.0, pos.y, pos.z+1.0);
	tmp.ba = getFrac(pos.x+1.0, pos.y, pos.z-1.0);
	tmp2.rg = getFrac(pos.x-1.0, pos.y, pos.z+1.0);
	tmp2.ba = getFrac(pos.x-1.0, pos.y, pos.z-1.0);
	flags = vec4(tmp.ga, tmp2.ga);

#if REMOVE_LONELY_INTERFACE_CELLS
	if (fluid_fraction.g == -1.0)
	{
#endif
		if (any(equal(flags, vec4(FLAGS_INTERFACE))))
		{
			frac = vec4(tmp.rb, tmp2.rb);
			if (any(greaterThan(frac, vec4(1.0 + EXTRA_OFFSET))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_INTERFACE,0,0);		// ... then this becomes an interface => reconstruct DF's
#else
				gl_FragColor = vec4(0,0,0,0);		// ... then this becomes an interface => reconstruct DF's
#endif
				return;
			}
		}
#if REMOVE_LONELY_INTERFACE_CELLS
	}
	else
	{
		anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(-1)));
		anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(+1)));
	}
#endif

	tmp.rg = getFrac(pos.x, pos.y+1.0, pos.z+1.0);
	tmp.ba = getFrac(pos.x, pos.y+1.0, pos.z-1.0);
	tmp2.rg = getFrac(pos.x, pos.y-1.0, pos.z+1.0);
	tmp2.ba = getFrac(pos.x, pos.y-1.0, pos.z-1.0);

	flags = vec4(tmp.ga, tmp2.ga);
#if REMOVE_LONELY_INTERFACE_CELLS
	if (fluid_fraction.g == -1.0)
	{
#endif
		if (any(equal(flags, vec4(FLAGS_INTERFACE))))
		{
			frac = vec4(tmp.rb, tmp2.rb);
			if (any(greaterThan(frac, vec4(1.0 + EXTRA_OFFSET))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_INTERFACE,0,0);		// ... then this becomes an interface => reconstruct DF's
#else
				gl_FragColor = vec4(0,0,0,0);		// ... then this becomes an interface => reconstruct DF's
#endif
				return;
			}
		}
#if REMOVE_LONELY_INTERFACE_CELLS
	}
	else
	{
		anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(-1)));
		anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(+1)));
	}
#endif

	tmp.rg = getFrac(pos.x, pos.y, pos.z+1.0);
	tmp.ba = getFrac(pos.x, pos.y, pos.z-1.0);

	flags.rg = tmp.ga;
#if REMOVE_LONELY_INTERFACE_CELLS
	if (fluid_fraction.g == -1.0)
	{
#endif
		if (any(equal(flags.rg, vec2(FLAGS_INTERFACE))))
		{
			frac.rg = tmp.rb;
			if (any(greaterThan(frac.rg, vec2(1.0 + EXTRA_OFFSET))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_INTERFACE,0,0);		// ... then this becomes an interface => reconstruct DF's
#else
				gl_FragColor = vec4(0,0,0,0);		// ... then this becomes an interface => reconstruct DF's
#endif
				return;
			}
		}
#if REMOVE_LONELY_INTERFACE_CELLS
	}
	else
	{
		anyGasNeighbor = anyGasNeighbor || any(equal(flags.rg, vec2(-1)));
		anyFluidNeighbor = anyFluidNeighbor || any(equal(flags.rg, vec2(+1)));
/*
		if (anyGasNeighbor && !anyFluidNeighbor && remove)
		{
			// we cannot remove this interface cell because an interface can become a fluid in this render step!
			// if there is an adjacent gas cell and no adjacent fluid neighbor, delete this 
			gl_FragColor = vec4(0, -1, 0, 0);	// => convert to gas
			return;
		}
*/

		if (!anyGasNeighbor && anyFluidNeighbor)
		{
			gl_FragColor = vec4(fluid_fraction.r, +1, 0, 0);	// => convert to fluid
			return;
		}
	}
#endif


	gl_FragColor = fluid_fraction;
	return;
}
