#extension GL_ARB_texture_rectangle : enable
#extension GL_ARB_draw_buffers : enable 

#include "shader/config.inc"


uniform vec3 res;
uniform vec3 res_sub_1;
uniform float ft_z_mod;

uniform sampler2DRect TextureFluidFraction;	// the new fluid fractions include the fluid fractions and the obstacle data as the 2 flag

/**
 * get fraction data (fluid fraction + flag)
 */
vec2 getFrac(float x, float y, float z)
{
	vec3 p = clamp(vec3(x,y,z), vec3(0.0), res_sub_1);

	vec2 tmpv = vec2(mod(p.z, ft_z_mod), floor(p.z / ft_z_mod));
	return texture2DRect(TextureFluidFraction, p.xy + tmpv*res.xy).rg;
}


void main(void)
{
	vec2 pr = floor(vec2(gl_TexCoord[0]));	// rectangle coordinates
	vec4 fluid_fraction = texture2DRect(TextureFluidFraction, pr);

	vec3 pos;	// 3d position
	pos.xy = floor(pr.xy/res.xy);
	pos.z = pos.x + pos.y * ft_z_mod;
	pos.xy = mod(pr.xy, res.xy);

#if DEBUG_SHADER
	if (fluid_fraction.g == 4.0)
	{
		gl_FragColor = fluid_fraction;
		return;
	}
#endif

/*
 * GAS + new GAS INTERFACE
 * these values are not modified by this shader
 */
	if (fluid_fraction.g == FLAGS_GAS || fluid_fraction.g == -10.0)
	{
		gl_FragColor = fluid_fraction;
		return;
	}

/*
 * INTERFACE CELL
 * 	=> GAS if fluid_fraction < -EXTRA_OFFSET
 */
	if (fluid_fraction.g == FLAGS_INTERFACE)
	{
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.r < -EXTRA_OFFSET)
		{
			gl_FragColor = vec4(fluid_fraction.r, FLAGS_GAS, 0, 0);	// new gas
			return;
		}	// after this we analyze if we can delete this interface cell
#else
		if (fluid_fraction.r < -EXTRA_OFFSET)
			gl_FragColor = vec4(fluid_fraction.r, FLAGS_GAS, 0, 0);
		else
			gl_FragColor = fluid_fraction;
		return;

#endif
	}


#if DEBUG_SHADER
	if (fluid_fraction.g != FLAGS_FLUID && fluid_fraction.g != FLAGS_OBSTACLE && fluid_fraction.g != FLAGS_GAS)
	{
		gl_FragColor = vec4(0, 4, 0, 4);
		return;
	}
#endif

/*
 * now we work on FLUID and OBSTACLE CELLS
 * check for neighbored conversion to interface cell
 *
 * also check the adjacent cells of obstacle cells.
 * if they converted to gas interface cells the DFs also have to be
 * reconstructed on obstacle cells
 */
	vec4 tmp, tmp2, flags, frac;

	tmp.rg = getFrac(pos.x-1.0, pos.y, pos.z);
	tmp.ba = getFrac(pos.x+1.0, pos.y, pos.z);
	tmp2.rg = getFrac(pos.x, pos.y-1.0, pos.z);
	tmp2.ba = getFrac(pos.x, pos.y+1.0, pos.z);
	flags = vec4(tmp.ga, tmp2.ga);

#if REMOVE_LONELY_INTERFACE_CELLS
	bool anyGasNeighbor;
	bool anyFluidNeighbor;
#endif
	if (fluid_fraction.g == FLAGS_FLUID)
	{
		if (any(equal(flags, vec4(FLAGS_INTERFACE))))
		{
			// neighbored interface cell
			frac = vec4(tmp.rb, tmp2.rb);
			if (any(lessThan(frac, vec4(-EXTRA_OFFSET))))	// neighbor is going to be a gas cell
			{
				// one of the neighbored cells becomes a gas cell
				gl_FragColor = vec4(1,FLAGS_INTERFACE,0,0);	// create interface cell from fluid
				return;
			}
		}
	}
	else
	{
		// handle OBSTACLE cells
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.g == FLAGS_OBSTACLE)
		{
#endif
			// obstacle - search for neighbored new gas cell
			if (any(equal(flags, vec4(FLAGS_NEW_INTERFACE))))		// neighbored cell became new interface cell from gas
									// therefore we also have to reconstruct the DFs at the obstacle cells
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_OBSTACLE,0,0);		// => reconstruct DF's
#else
				gl_FragColor = vec4(0,FLAGS_INTERFACE,0,0);		// => reconstruct DF's
#endif
				return;
			}
#if REMOVE_LONELY_INTERFACE_CELLS
		}
		else
		{
			anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(FLAGS_GAS)));
			anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(FLAGS_FLUID)));
		}
#endif
	}

	tmp.rg = getFrac(pos.x+1.0, pos.y-1.0, pos.z);
	tmp.ba = getFrac(pos.x+1.0, pos.y+1.0, pos.z);
	tmp2.rg = getFrac(pos.x-1.0, pos.y+1.0, pos.z);
	tmp2.ba = getFrac(pos.x-1.0, pos.y-1.0, pos.z);

	flags = vec4(tmp.ga, tmp2.ga);
	frac = vec4(tmp.rb, tmp2.rb);
	if (fluid_fraction.g == FLAGS_FLUID)
	{
		// fluid - search for cells converting to gas
		if (any(equal(flags, vec4(FLAGS_INTERFACE))))
		{
			if (any(lessThan(frac, vec4(-EXTRA_OFFSET))))
			{
				gl_FragColor = vec4(1,FLAGS_INTERFACE,0,0);
				return;
			}
		}
	}
	else
	{
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.g == FLAGS_OBSTACLE)
		{
#endif
			// obstacle - search for neighbored new gas cell
			if (any(equal(flags, vec4(FLAGS_NEW_INTERFACE))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_OBSTACLE,0,0);		// => reconstruct DF's
#else
				gl_FragColor = vec4(0,FLAGS_INTERFACE,0,0);		// => reconstruct DF's
#endif
				return;
			}
#if REMOVE_LONELY_INTERFACE_CELLS
		}
		else
		{
			anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(FLAGS_GAS)));
			anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(FLAGS_FLUID)));
		}
#endif
	}


	tmp.rg = getFrac(pos.x+1.0, pos.y, pos.z+1.0);
	tmp.ba = getFrac(pos.x+1.0, pos.y, pos.z-1.0);
	tmp2.rg = getFrac(pos.x-1.0, pos.y, pos.z+1.0);
	tmp2.ba = getFrac(pos.x-1.0, pos.y, pos.z-1.0);
	flags = vec4(tmp.ga, tmp2.ga);

	frac = vec4(tmp.rb, tmp2.rb);
	if (fluid_fraction.g == FLAGS_FLUID)
	{
		// fluid - search for cells converting to gas
		if (any(equal(flags, vec4(0))))
		{
			if (any(lessThan(frac, vec4(-EXTRA_OFFSET))))
			{
				gl_FragColor = vec4(1,FLAGS_INTERFACE,0,0);
				return;
			}
		}
	}
	else
	{
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.g == FLAGS_OBSTACLE)
		{
#endif
			// obstacle - search for neighbored new gas cell
			if (any(equal(flags, vec4(FLAGS_NEW_INTERFACE))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_OBSTACLE,0,0);		// => reconstruct DF's
#else
				gl_FragColor = vec4(0,FLAGS_INTERFACE,0,0);		// => reconstruct DF's
#endif
				return;
			}
#if REMOVE_LONELY_INTERFACE_CELLS
		}
		else
		{
			anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(FLAGS_GAS)));
			anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(FLAGS_FLUID)));
		}
#endif
	}


	tmp.rg = getFrac(pos.x, pos.y+1.0, pos.z+1.0);
	tmp.ba = getFrac(pos.x, pos.y+1.0, pos.z-1.0);
	tmp2.rg = getFrac(pos.x, pos.y-1.0, pos.z+1.0);
	tmp2.ba = getFrac(pos.x, pos.y-1.0, pos.z-1.0);

	flags = vec4(tmp.ga, tmp2.ga);
	frac = vec4(tmp.rb, tmp2.rb);
	if (fluid_fraction.g == FLAGS_FLUID)
	{
		// fluid - search for cells converting to gas
		if (any(equal(flags, vec4(0))))
		{
			if (any(lessThan(frac, vec4(-EXTRA_OFFSET))))
			{
				gl_FragColor = vec4(1,FLAGS_INTERFACE,0,0);
				return;
			}
		}
	}
	else
	{
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.g == FLAGS_OBSTACLE)
		{
#endif
			// obstacle - search for neighbored new gas cell
			if (any(equal(flags, vec4(FLAGS_NEW_INTERFACE))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_OBSTACLE,0,0);		// => reconstruct DF's
#else
				gl_FragColor = vec4(0,FLAGS_INTERFACE,0,0);		// => reconstruct DF's
#endif
				return;
			}
#if REMOVE_LONELY_INTERFACE_CELLS
		}
		else
		{
			anyGasNeighbor = anyGasNeighbor || any(equal(flags, vec4(FLAGS_GAS)));
			anyFluidNeighbor = anyFluidNeighbor || any(equal(flags, vec4(FLAGS_FLUID)));
		}
#endif
	}

	tmp.rg = getFrac(pos.x, pos.y, pos.z-1.0);
	tmp.ba = getFrac(pos.x, pos.y, pos.z+1.0);

	flags.rg = tmp.ga;
	frac.rg = tmp.rb;
	if (fluid_fraction.g == FLAGS_FLUID)
	{
		// fluid - search for cells converting to gas
		if (any(equal(flags, vec4(0))))
		{
			if (any(lessThan(frac, vec4(-EXTRA_OFFSET))))
			{
				gl_FragColor = vec4(1,FLAGS_INTERFACE,0,0);
				return;
			}
		}
	}
	else
	{
#if REMOVE_LONELY_INTERFACE_CELLS
		if (fluid_fraction.g == FLAGS_OBSTACLE)
		{
#endif
			// obstacle - search for neighbored new gas cell
			if (any(equal(flags.rg, vec2(FLAGS_NEW_INTERFACE))))
			{
#if HANDLE_GAS
				gl_FragColor = vec4(0,FLAGS_NEW_OBSTACLE,0,0);		// => reconstruct DF's
#else
				gl_FragColor = vec4(0,FLAGS_INTERFACE,0,0);		// => reconstruct DF's
#endif
				return;
			}
#if REMOVE_LONELY_INTERFACE_CELLS
		}
		else
		{
			anyGasNeighbor = anyGasNeighbor || any(equal(flags.rg, vec2(FLAGS_GAS)));
			anyFluidNeighbor = anyFluidNeighbor || any(equal(flags.rg, vec2(FLAGS_FLUID)));

			if (anyGasNeighbor && !anyFluidNeighbor)
			{
				// if there is an adjacent gas cell and no adjacent fluid neighbor, delete this 
				gl_FragColor = vec4(fluid_fraction.r, FLAGS_GAS, 0, 0);	// => convert to gas
				return;
			}

/*
			// we cannot remove this an interface can become a gas in this step
			if (!anyGasNeighbor && anyFluidNeighbor)
			{
				gl_FragColor = vec4(1, +1, 0, 0);	// => convert to fluid
				return;
			}
*/
		}
#endif
	}

	gl_FragColor = fluid_fraction;
	return;
}
