#extension GL_ARB_texture_rectangle : enable
#include "shader/config.inc"

uniform vec3 res;
uniform vec3 res_sub_1;
uniform float ft_z_mod;

uniform sampler2DRect TextureFluidFraction;
uniform sampler2DRect TextureVelocityDensity;

/**
 * get velocity
 */
vec4 getVelocity(float x, float y, float z)
{
	vec3 p = clamp(vec3(x,y,z), vec3(0.0), res_sub_1);

	vec2 tmpv = vec2(mod(p.z, ft_z_mod), floor(p.z / ft_z_mod));
	return texture2DRect(TextureVelocityDensity, p.xy + tmpv*res.xy);
}

vec4 getFlag(float x, float y, float z)
{
	vec3 p = clamp(vec3(x,y,z), vec3(0.0), res_sub_1);

	vec2 tmpv = vec2(mod(p.z, ft_z_mod), floor(p.z / ft_z_mod));
	return texture2DRect(TextureFluidFraction, p.xy + tmpv*res.xy);
}

void main(void)
{
	vec2 pr = floor(vec2(gl_TexCoord[0]));	// rectangle coordinates
	vec4 fluid_fraction = texture2DRect(TextureFluidFraction, pr);

#if DEBUG_SHADER
	if (fluid_fraction.g == 4.0)
	{
		gl_FragColor = fluid_fraction;
		return;
	}
#endif

#if !HANDLE_GAS
	gl_FragColor = texture2DRect(TextureVelocityDensity, pr);
	return;
#endif


	vec3 pos;	// 3d position
	pos.xy = floor(pr.xy/res.xy);
	pos.z = pos.x + pos.y * ft_z_mod;
	pos.xy = mod(pr.xy, res.xy);


//	if (fluid_fraction.g != FLAGS_NEW_INTERFACE && fluid_fraction.g != FLAGS_NEW_INTERFACE)
	if (fluid_fraction.g > -10.0)	// reconstruct new fluid interfaces (-10) as well as obstacles neighbored to them (-20)
	{
		gl_FragColor = texture2DRect(TextureVelocityDensity, pr);
		return;
	}

	vec4 flag;
	vec4 velocity = vec4(0);
	float count= 0.0;

	// load the flags of the adjacent flags and check if there this is a fluid or interface cell
	flag.r = float(any(equal(getFlag(pos.x-1.0, pos.y, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.g = float(any(equal(getFlag(pos.x+1.0, pos.y, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.b = float(any(equal(getFlag(pos.x, pos.y-1.0, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.a = float(any(equal(getFlag(pos.x, pos.y+1.0, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	if (flag.r > 0.0)	{	velocity += getVelocity(pos.x-1.0, pos.y, pos.z);	count++;	}
	if (flag.g > 0.0)	{	velocity += getVelocity(pos.x+1.0, pos.y, pos.z);	count++;	}
	if (flag.b > 0.0)	{	velocity += getVelocity(pos.x, pos.y-1.0, pos.z);	count++;	}
	if (flag.a > 0.0)	{	velocity += getVelocity(pos.x, pos.y+1.0, pos.z);	count++;	}

	flag.r = float(any(equal(getFlag(pos.x+1.0, pos.y-1.0, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.g = float(any(equal(getFlag(pos.x+1.0, pos.y+1.0, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.b = float(any(equal(getFlag(pos.x-1.0, pos.y-1.0, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.a = float(any(equal(getFlag(pos.x-1.0, pos.y+1.0, pos.z).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	if (flag.r > 0.0)	{	velocity += getVelocity(pos.x+1.0, pos.y-1.0, pos.z);	count++;	}
	if (flag.g > 0.0)	{	velocity += getVelocity(pos.x+1.0, pos.y+1.0, pos.z);	count++;	}
	if (flag.b > 0.0)	{	velocity += getVelocity(pos.x-1.0, pos.y-1.0, pos.z);	count++;	}
	if (flag.a > 0.0)	{	velocity += getVelocity(pos.x-1.0, pos.y+1.0, pos.z);	count++;	}

	flag.r = float(any(equal(getFlag(pos.x+1.0, pos.y, pos.z-1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.g = float(any(equal(getFlag(pos.x+1.0, pos.y, pos.z+1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.b = float(any(equal(getFlag(pos.x-1.0, pos.y, pos.z-1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.a = float(any(equal(getFlag(pos.x-1.0, pos.y, pos.z+1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	if (flag.r > 0.0)	{	velocity += getVelocity(pos.x+1.0, pos.y, pos.z-1.0);	count++;	}
	if (flag.g > 0.0)	{	velocity += getVelocity(pos.x+1.0, pos.y, pos.z+1.0);	count++;	}
	if (flag.b > 0.0)	{	velocity += getVelocity(pos.x-1.0, pos.y, pos.z-1.0);	count++;	}
	if (flag.a > 0.0)	{	velocity += getVelocity(pos.x-1.0, pos.y, pos.z+1.0);	count++;	}

	flag.r = float(any(equal(getFlag(pos.x, pos.y+1.0, pos.z-1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.g = float(any(equal(getFlag(pos.x, pos.y+1.0, pos.z+1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.b = float(any(equal(getFlag(pos.x, pos.y-1.0, pos.z-1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.a = float(any(equal(getFlag(pos.x, pos.y-1.0, pos.z+1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	if (flag.r > 0.0)	{	velocity += getVelocity(pos.x, pos.y+1.0, pos.z-1.0);	count++;	}
	if (flag.g > 0.0)	{	velocity += getVelocity(pos.x, pos.y+1.0, pos.z+1.0);	count++;	}
	if (flag.b > 0.0)	{	velocity += getVelocity(pos.x, pos.y-1.0, pos.z-1.0);	count++;	}
	if (flag.a > 0.0)	{	velocity += getVelocity(pos.x, pos.y-1.0, pos.z+1.0);	count++;	}

	flag.r = float(any(equal(getFlag(pos.x, pos.y, pos.z-1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	flag.g = float(any(equal(getFlag(pos.x, pos.y, pos.z+1.0).gg, vec2(FLAGS_FLUID, FLAGS_INTERFACE))));
	if (flag.r > 0.0)	{	velocity += getVelocity(pos.x, pos.y, pos.z-1.0);	count++;	}
	if (flag.g > 0.0)	{	velocity += getVelocity(pos.x, pos.y, pos.z+1.0);	count++;	}

	if (count == 0.0)	velocity = vec4(0,0,0,1);
	else			velocity /= count;

	gl_FragColor = velocity;
}
