#extension GL_ARB_texture_rectangle : enable
#extension GL_ARB_draw_buffers : enable

/**
 * gl_FragData[0]: velocity, density
 * gl_FragData[1]: fluid fraction
 */

uniform sampler2DRect Texture0, Texture1, Texture2, Texture3, Texture4;
uniform sampler2DRect TextureFluidMass, TextureFluidFraction;

void main(void)
{
	vec4 v = vec4(0);
	vec2 tmpv;

	vec2 pr = floor(vec2(gl_TexCoord[0]));
	vec4 f;

	f = texture2DRect(Texture0, pr);
	tmpv = f.rg + f.ba;		// density
	v.a += tmpv.r+tmpv.g;
	v.rg += f.rb - f.ga;
/*
	v.r += f.r;			// f(1,0,0)
	v.r -= f.g;			// f(-1,0,0)
	v.g += f.b;			// f(0,1,0)
	v.g -= f.a;			// f(0,-1,0)
*/

	f = texture2DRect(Texture1, pr);
	tmpv = f.rg + f.ba;		// density
	v.a += tmpv.r+tmpv.g;
	v.rg += f.rr - f.gg + f.ba - f.ab;
/*
	v.r += f.r;	v.g += f.r;	// f(1,1,0)
	v.r -= f.g;	v.g -= f.g;	// f(-1,-1,0)
	v.r += f.b;	v.g -= f.b;	// f(1,-1,0)
	v.r -= f.a;	v.g += f.a;	// f(-1,1,0)
*/
	f = texture2DRect(Texture2, pr);
	tmpv = f.rg + f.ba;		// density
	v.a += tmpv.r+tmpv.g;
	v.rb += f.rr - f.gg + f.ba - f.ab;
/*
	v.r += f.r;	v.b += f.r;	// f(1,0,1)
	v.r -= f.g;	v.b -= f.g;	// f(-1,0,-1)
	v.r += f.b;	v.b -= f.b;	// f(1,0,-1)
	v.r -= f.a;	v.b += f.a;	// f(-1,0,1)
*/
	f = texture2DRect(Texture3, pr);
	tmpv = f.rg + f.ba;		// density
	v.a += tmpv.r+tmpv.g;
	v.gb += f.rr - f.gg + f.ba - f.ab;
/*
	v.g += f.r;	v.b += f.r;	// f(0,1,1)
	v.g -= f.g;	v.b -= f.g;	// f(0,-1,-1)
	v.g += f.b;	v.b -= f.b;	// f(0,1,-1)
	v.g -= f.a;	v.b += f.a;	// f(0,-1,1)
*/
	f = texture2DRect(Texture4, pr);
	v.a += f.r + f.g + f.b;		// density
	v.b += f.r;			// f(0,0,1)
	v.b -= f.g;			// f(0,0,-1)
	;				// f(0,0,0)
	;				// obstacle information in alpha channel

	gl_FragData[0] = v;	// store velocity and density

/*
 * computate new FLUID FRACTION
 * this data is used to move the interface cells
 */
	if (f.a > 0.0)
	{
		// set obstacle data (no fluid fraction)
		gl_FragData[1] = vec4(0, 2, 0, 1);
		return;
	}

	float fluid_mass = texture2DRect(TextureFluidMass, pr).r;
	gl_FragData[1] = vec4(fluid_mass/v.a, texture2DRect(TextureFluidFraction, pr).gba);
}
