#extension GL_ARB_texture_rectangle : enable

/**
 * combine the slices created by depth peeling to a flat voxel texture
 */

uniform vec3 res, inv_res;
uniform vec2 texture_size;
uniform vec2 texture_size_x, texture_size_y, texture_size_z;	// texture sizes for specified axis
uniform vec2 inv_texture_size_x, inv_texture_size_y, inv_texture_size_z;	// texture sizes for specified axis

uniform float tex_z_mod, tex_y_mod, tex_x_mod;	// count of slices aligned in texture x axis
uniform float inv_tex_z_mod, inv_tex_y_mod, inv_tex_x_mod;

uniform sampler2DRect TextureZ, TextureY, TextureX;	// 0 -> Z Slices of depth peeling
						// 1 -> Y Slices of depth peeling
						// 2 -> X Slices of depth peeling

uniform float max_z_slices, max_y_slices, max_x_slices;

void main(void)
{
	vec2 pr = floor(vec2(gl_TexCoord[0]));
	vec3 p;
	float depth;

	vec2 tmp = floor((pr.xy + 0.5)*inv_res.xy);
	vec3 pos = vec3(
		mod(pr.xy, res.xy),
		tmp.x + tmp.y * tex_z_mod
	);

	// now we have to walk through all z depth peeling slices (maximum of max_z_slices)
	p.xy = pos.xy;		// point position on the depth peeling texture
	// fix the near/front plane which is set to -1, 1 to avoid clipping
	// use this var to avoid computations within loop
	float checkvarz = pos.z + res.z*0.5;
	for (p.z = 0.0; p.z < max_z_slices; p.z++)
	{
		if (p.x >= texture_size_z.x)
		{
			p.x = pos.x;
			p.y += res.y;
		}

		vec4 c = texture2DRect(TextureZ, p.xy);
		depth = c.a*2.0*res.z;

		if (checkvarz+1.5 < depth)	break;	// no more depth slices

		if (	checkvarz-0.5 < depth	&&
			checkvarz+1.5 > depth
		)
		{
			gl_FragColor = vec4(c.rgb, 1.0);
			return;
		}

		p.x += res.x;
	}

	// and the y slices...
	p.xz = pos.xz;
	float checkvary = pos.y + res.y*0.5;
	for (p.y = 0.0; p.y < max_y_slices; p.y++)
	{
		if (p.x >= texture_size_y.x)
		{
			p.x = pos.x;
			p.z += res.z;
		}

		vec4 c = texture2DRect(TextureY, p.xz);
		depth = c.a*2.0*res.y;

		if (checkvary+1.5 < depth)	break;	// no more depth slices

		if (	checkvary-0.5 < depth	&&
			checkvary+1.5 > depth
		)
		{
			gl_FragColor = vec4(c.rgb, 1.0);
			return;
		}

		p.x += res.x;
	}

	// and the x slices... (y along x axis, Z along y axis)
	p.yz = pos.yz;
	float checkvarx = pos.x + res.x*0.5;
	for (p.x = 0.0; p.x < max_x_slices; p.x++)
	{
		if (p.y >= texture_size_x.x)
		{
			p.y = pos.y;
			p.z += res.z;
		}

		vec4 c = texture2DRect(TextureX, p.yz);
		depth = c.a*2.0*res.x;

		if (checkvarx+1.5 < depth)	break;	// no more depth slices

		if (	checkvarx-0.5 < depth	&&
			checkvarx+1.5 > depth
		)
		{
			gl_FragColor = vec4(c.rgb, 1.0);
			return;
		}

		p.y += res.y;
	}

	gl_FragColor = vec4(0.0);
	return;
}
