/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GUI_H
#define GUI_H


#include <stdio.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <GL/glew.h>
#include <GL/gl.h>
#include "libviz/gltools.hpp"
#include <string>
using namespace std;

// shift pressed
#define KEY_MOD_SHIFT	KMOD_SHIFT

class CGui
{
public:
	float screen_aspect;
	int screen_height, screen_width;

	bool fullscreen_active;

	// CALLBACK HANDLER
	void *callback_class;
	void (*callback_key_down)(void*, int key, int mod, int scancode, int unicode);	// key event
	void (*callback_key_up)(void*, int key, int mod, int scancode, int unicode);	// key event
	void (*callback_quit)(void*);		// quit event
	void (*callback_mouse_motion)(void*, int x, int y);	// mouse position in absolute coordinates
	void (*callback_mouse_button_down)(void*, int button);	// button press event
	void (*callback_mouse_button_up)(void*, int button);	// button release event
	void (*callback_viewport_changed)(void*, int width, int height);

public:
	/**
	 * deactivate rotation with mouse
	 * e.g. if the mouse draws on something
	 */
	void deactivate_mouse_rotation();

	/**
	 * activate mouse rotation again
	 */
	void activate_mouse_rotation();

	/**
	 * call this function for event handling
	 */
	void event_loop();

	/**
	 * call at begin of rendering
	 */
	void init_rendering();

	/**
	 * call at end of rendering
	 */
	void finish_rendering();

	/**
	 * set video mode to windowed or fullscreen
	 */
	void set_video_mode(bool fullscreen);

	/** 
	 * initialize gui
	 */
	//void init_gui();
	CGui();

	/**
	 * shutdown gui
	 */
	//void shutdown_gui();
	~CGui();

	void setup_viewport();
	signed int get_ticks();
	bool takeScreenshot(string &filename);

private:
	// size / aspect of sdl surface in window mode
	float window_aspect;
	int window_height, window_width;

	// size / aspect of sdl surface in fullscreen mode
	float fullscreen_aspect;
	int fullscreen_height, fullscreen_width;
};

#endif
