/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "CBitmap.hpp"
#include <stdio.h>

#include <string>
using namespace std;

CBitmap24::CBitmap24()	:
	data(NULL),
	error(false)
{
}


CBitmap24::CBitmap24(int p_width, int p_height)	:
	width(p_width),
	height(p_height),
	error(false)
{
	data = new char[width*height*3];
}
/*
void CBitmap24::loadFromMemory(int width, int height, char* data)
{
}
*/

/**
 * http://en.wikipedia.org/wiki/BMP_file_format
 */
bool CBitmap24::save(string &filename)
{
	FILE *file = fopen(filename.c_str(), "w");
	if (!file)
	{
		errorLog = string() + "cannot open file '" + filename.c_str() + "'";
		return false;
	}

	struct
	{
		unsigned short magic_number;
	} file_header_1;

	struct
	{
		unsigned int file_size;
		unsigned short res1;
		unsigned short res2;
		unsigned int offset_data;
	} file_header_2;

	int padding = (4-(3*width)%4)%4;

	file_header_1.magic_number = 'B'+'M'*256;
	file_header_2.file_size = 54+(3*width+padding)*height;
	file_header_2.res1 = 0;
	file_header_2.res2 = 0;
	file_header_2.offset_data = 54;

	int x;
	x = fwrite(&file_header_1, sizeof(file_header_1), 1, file);
	x = fwrite(&file_header_2, sizeof(file_header_2), 1, file);

	struct
	{
		unsigned int header_size;
		unsigned int width;
		unsigned int height;
		unsigned short color_planes;
		unsigned short bpp;
		unsigned int compression_method;
		unsigned int image_size;
		unsigned int h_res;
		unsigned int v_res;
		unsigned int colors_in_palette;
		unsigned int important_colors;
	} image_header;

	image_header.header_size = 40;
	image_header.width = width;
	image_header.height = height;
	image_header.color_planes = 1;
	image_header.bpp = 24;
	image_header.compression_method = 0;
	image_header.image_size = width*height*3;
	image_header.h_res = 11811;
	image_header.v_res = 11811;
	image_header.colors_in_palette = 0;
	image_header.important_colors = 0;

	x = fwrite(&image_header, sizeof(image_header), 1, file);

	// The rows in BMPs always contain a multiple of four pixels, so pad at the end with black pixels.
	// The resolution, however, may be arbitrary. That's what the iPad is good for.
	// write data
	if (padding == 0)
	{
		x = fwrite(data, 3*width*height, 1, file);
	}
	else
	{
		for (int y = 0; y < height; y++)
		{
			x = fwrite(&data[y*width*3], 3*width, 1, file);
			x = fwrite(data, padding, 1, file);
		}
	}

	fclose(file);

	return true;
}

CBitmap24::~CBitmap24()
{
	if (data)
		delete data;
}
