/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * version 0.2
 *
 * history:
 *
 * 2007-09-02: implemented CTexture class instead of static function
 */

#include <GL/glew.h>
#include <SDL/SDL.h>
#include <GL/glu.h>
#include <string.h>
#include <stdarg.h>
#include "CFont.hpp"
#include "gltools.hpp"
#include <iostream>


using namespace std;

/*
 * this class uses a texture (16 x 16 chars) to draw a string to the screen
 */
CFont::CFont(const char *file)
{
	fontTexture = new CTexture(file);

	if (fontTexture->valid == false)
	{
		cerr << "failed to load fontfile '" << file << "'" << endl;
		init = 0;
		return;
	}

	charWidth = (float)fontTexture->width / 16.0f;
	charHeight = (float)fontTexture->height / 16.0f;

	fontSize(charHeight);

	fontDisplayList = glGenLists(256);
	if (fontDisplayList == 0)
	{
		init = 0;
		return;
	}

	init = 1;

	fontTexture->bind();

	float x, y;
	for (int pos = 0; pos < 256; pos++)
	{
		x = ((float)(pos & 0x0f)) / 16.0f;
		y = ((float)(pos >> 4)) / 16.0f;

		glNewList(fontDisplayList + pos, GL_COMPILE);

		glBegin(GL_QUADS);
		  glTexCoord2f(x, y);					glVertex2d(0, 1);	// top left
		  glTexCoord2f(x, y+(1.0f/16.0f));			glVertex2d(0, 0);	// bottom left
		  glTexCoord2f(x+(1.0f/16.0f), y+(1.0f/16.0f));		glVertex2d(-1, 0);	// bottom right
		  glTexCoord2f(x+(1.0f/16.0f), y);			glVertex2d(-1, 1);	// top right
		glEnd();
//		glTranslatef(-charWidth, 0, 0);
		glTranslatef(-1, 0, 0);

		glEndList();
	}
}


void CFont::fontSize(float size)
{
	scaleY = size;
	scaleX = charWidth * (scaleY / charHeight);
}



void CFont::printfxy(int x, int y, const char *format, ...)
{
	va_list list;
	static char text[2048];
	float viewport[4];
	GLuint EDepth, EBlend, ETexture, ELight;

	if (init == 0)	return;

	va_start(list, format);
	vsnprintf(text, 2048, format, list);

	EDepth = glIsEnabled(GL_DEPTH_TEST);
	EBlend = glIsEnabled(GL_BLEND);
	ETexture = glIsEnabled(GL_TEXTURE_2D);
	ELight = glIsEnabled(GL_LIGHTING);

	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);

	glEnable(GL_BLEND);
	glBlendEquation(GL_FUNC_ADD);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glGetFloatv(GL_VIEWPORT, viewport);
	glOrtho(viewport[0], -viewport[2], viewport[1], viewport[3], -1.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	fontTexture->bind();

	glTranslatef(-x, viewport[3] - y - charHeight, 0.0);
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);	// for glTranslate in displaylist
	glScalef(scaleX, scaleY, 1);

	glListBase(fontDisplayList - 32);
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
	glListBase(0);

	fontTexture->unbind();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	if (EDepth)	glEnable(GL_DEPTH_TEST);
	if (EBlend)	glEnable(GL_BLEND);
	if (!ETexture)	glDisable(GL_TEXTURE_2D);
	if (ELight)	glEnable(GL_LIGHTING);

}
