/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "CObjFile.hpp"
#include <iostream>
#include "../libmath/CVector.hh"

using namespace std;

/*
 * store vertex da
 * if (o->vertex == 0), just return the vertex count in the file
 */
bool CObjFile::readData(bool storeData)
{
	char buffer[1024];
	char type;
	vec3f v_;
	int f3_[3];
	int f4_[4];

	CVector<3,float> *v;
	int *f3, *f4;
	obj_group_s *g = 0;

	int vc = 0;
	int gc = 0;
	int fc4 = 0;
	int fc3 = 0;

	int line = 0;

	if (!storeData)	
	{
		v = &v_;
		f3 = f3_;
		f4 = f4_;
	}
	else
	{
		v = vertices;
		f4 = faces4;
		f3 = faces3;
		g = 0;
	}

	fseek(file, 0, SEEK_SET);
	while(fgets(buffer, 1024, file) != NULL)
	{
		line++;

		if (strncmp(buffer, "v ", 2) == 0)
		{
			if (sscanf(buffer, "%c %f %f %f\n", &type, &(*v)[0], &(*v)[1], &(*v)[2]) == 4)
			{
				if (storeData)
					v++;
			}
			else
			{
				fprintf(stderr, "error in line %i\n", line);
				return false;
			}
			vc++;
		}
		else if (strncmp(buffer, "g ", 2) == 0)
		{
			char *c;
			c = strchr(buffer, '\n');
			if (c != NULL)	*c = '\0';

			// read out parameter, if groupname is in the format [name]_[int]_[float]
			char *int_c;
			int int_value = -1;

			char *float_c;
			float float_value = -1;

			c = strchr(buffer, '_');
			if (c != NULL)
			{
				*c = '\0';
				int_c = c+1;

				c = strchr(int_c, '_');
				if (c != NULL)
				{
					*c = '\0';
					float_c = c+1;

					c = strchr(float_c, '_');
					if (c != NULL)	*c = '.';

					float_value = atof(float_c);
				}
				int_value = atoi(int_c);
			}

			if (storeData)
			{
				if (g == NULL)	g = groups;
				else		g++;

				g->name = strdup(&buffer[2]);
				g->int_param = int_value;
				g->float_param = float_value;
				g->faces3 = f3;
				g->faces4 = f4;
				g->faces3_count = 0;
				g->faces4_count = 0;
			}
			else
			{
				printf("Group[%s, %i, %f]\n", &buffer[2], int_value, float_value);
			}
			gc++;
		}
		else if (strncmp(buffer, "f ", 2) == 0)
		{
			int tmp;
			char *c;

			int count;
			c = buffer + 2;
			for (count = 0; c; c = strchr(c, ' '))
			{
				count++;
				c++;
			}

			if (count == 4)
			{
				if (!(sscanf(buffer, "%c %i/%i/%i %i/%i/%i %i/%i/%i %i/%i/%i\n",
							&type,
							&f4[0], &tmp, &tmp,
							&f4[1], &tmp, &tmp,
							&f4[2], &tmp, &tmp,
							&f4[3], &tmp, &tmp
					  ) == 13))
				if (!(sscanf(buffer, "%c %i//%i %i//%i %i//%i %i//%i\n",
							&type,
							&f4[0], &tmp,
							&f4[1], &tmp,
							&f4[2], &tmp,
							&f4[3], &tmp
					  ) == 9))
				if (!(sscanf(buffer, "%c %i/%i %i/%i %i/%i %i/%i\n",
							&type,
							&f4[0], &tmp,
							&f4[1], &tmp,
							&f4[2], &tmp,
							&f4[3], &tmp
					  ) == 9))
				if (!(sscanf(buffer, "%c %i %i %i %i\n",
								&type,
								&f4[0],
								&f4[1],
								&f4[2],
								&f4[3]
						  ) == 5))
				{
					fprintf(stderr, "error in line %i\n", line);
					return false;
				}

				f4[0]--;	f4[1]--;	f4[2]--;	f4[3]--;

				if (	f4[0] > vc	||	f4[0] < 0	||
					f4[1] > vc	||	f4[1] < 0	||
					f4[2] > vc	||	f4[2] < 0	||
					f4[3] > vc	||	f4[3] < 0
				)
				{
					fprintf(stderr, "invalid face in line %i\n", line);
					return false;
				}

				if (storeData)
				{
					f4 += 4;
					if (g)	g->faces4_count++;
				}
				else
				{
//					printf("F4[%i] = {%i, %i, %i, %i}\n", fc4, f4[0], f4[1], f4[2], f4[3]);
				}
				fc4++;
			}
			else if (count == 3)
			{
				if (!(sscanf(buffer, "%c %i/%i/%i %i/%i/%i %i/%i/%i\n",
							&type,
							&f3[0], &tmp, &tmp,
							&f3[1], &tmp, &tmp,
							&f3[2], &tmp, &tmp
					  ) == 10))
				if (!(sscanf(buffer, "%c %i//%i %i//%i %i//%i\n",
							&type,
							&f3[0], &tmp,
							&f3[1], &tmp,
							&f3[2], &tmp
					  ) == 7))
				if (!(sscanf(buffer, "%c %i/%i %i/%i %i/%i\n",
							&type,
							&f3[0], &tmp,
							&f3[1], &tmp,
							&f3[2], &tmp
					  ) == 7))
				if (!(sscanf(buffer, "%c %i %i %i\n",
								&type,
								&f4[0],
								&f4[1],
								&f4[2]
						  ) == 4))
				{
					fprintf(stderr, "error in line %i\n", line);
					return false;
				}

				f3[0]--;	f3[1]--;	f3[2]--;

				if (	f3[0] > vc	||	f3[0] < 0	||
					f3[1] > vc	||	f3[1] < 0	||
					f3[2] > vc	||	f3[2] < 0
				)
				{
					fprintf(stderr, "invalid face in line %i\n", line);
					return false;
				}

				if (storeData)
				{
					f3 += 3;
					if (g)	g->faces3_count++;
				}
				else
				{
//					printf("F3[%i] = {%i, %i, %i}\n", fc3, f3[0], f3[1], f3[2]);
				}
				fc3++;
			}
		}
	}

	if (!storeData)
	{
		vertex_count = vc;
		group_count = gc;
		faces4_count = fc4;
		faces3_count = fc3;
	}
	else
	{
		printf("%i Vertex, %i Groups, %i 4 Faces, %i 3 Faces\n", vc, gc, fc4, fc3);
	}

	return true;
}


/**
 * load data from .obj file
 * \param objfile	path to .obj file
 */
CObjFile::CObjFile(char *objfile)
{
	ready = false;

	vertices = 0;
	faces4 = 0;
	faces3 = 0;
	groups = 0;

	file = fopen(objfile, "r");
	if (!file)	return;

	if (!readData(false))
	{
		fclose(file);
		return;
	}

	vertices = new CVector<3,float>[vertex_count];
	normals = new CVector<3,float>[vertex_count];
	faces4 = new int[faces4_count*4];
	faces3 = new int[faces3_count*3];
	groups = new obj_group_s[group_count];

	readData(true);
	fclose(file);

	// calculate normals

	CVector<3,float> *n = normals;
	for (int i = 0; i < vertex_count; i++)
	{
		n->setZero();
		n++;
	}

	int *f;
	vec3f a, b, norm;

	f = faces3;
	for (int i = 0; i < faces3_count; i++)
	{
		CVector<3,float> &v0 = vertices[f[0]];
		CVector<3,float> &v1 = vertices[f[1]];
		CVector<3,float> &v2 = vertices[f[2]];

		a = v1 - v0;
		b = v2 - v0;

		norm[0] = a[1]*b[2] - a[2]*b[1];
		norm[1] = a[2]*b[0] - a[0]*b[2];
		norm[2] = a[0]*b[1] - a[1]*b[0];

		normals[f[0]] += norm;
		normals[f[1]] += norm;
		normals[f[2]] += norm;

		f += 3;
	}

	f = faces4;
	for (int i = 0; i < faces4_count; i++)
	{
		CVector<3,float> &v0 = vertices[f[0]];
		CVector<3,float> &v1 = vertices[f[1]];
		CVector<3,float> &v2 = vertices[f[2]];

		a = v1 - v0;
		b = v2 - v0;

		norm[0] = a[1]*b[2] - a[2]*b[1];
		norm[1] = a[2]*b[0] - a[0]*b[2];
		norm[2] = a[0]*b[1] - a[1]*b[0];

		normals[f[0]] += norm;
		normals[f[1]] += norm;
		normals[f[2]] += norm;
		normals[f[3]] += norm;

		f += 4;
	}

	n = normals;
	for (int i = 0; i < vertex_count; i++)
	{
		n->normalize();
		n++;
	}

	ready = true;
	printf("Objfile read\n");
}


/**
 * cleanup all allocated
 */
CObjFile::~CObjFile()
{
	if (vertices)
	{
		delete vertices;
		delete normals;
		delete faces3;
		delete faces4;
		delete groups;
	}
}

