/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file CObjFile.hpp
 * \brief load object data from an .obj file
 *
 * \class CObjFile
 * \brief load object data from an .obj file
 */

#ifndef __OBJFILE_H
#define __OBJFILE_H


#include <stdio.h>
#include "../libmath/CVector.hh"



struct obj_group_s
{
	char *name;

	int int_param;
	float float_param;

	int *faces4;		// 1 surface has 4 index values to vertex points
	int faces4_count;

	int *faces3;		// 1 surface has 3 index values to vertex points
	int faces3_count;
};


class CObjFile
{
private:
	FILE *file;
	bool readData(bool readData);

public:
	bool ready;		///< loading of object file was successful

	CVector<3,float> *vertices;
	CVector<3,float> *normals;
//	float *vertex;		///< vertex data  - 1 point has 3 floats
//	float *normals;		///< normals calculated for vertex
	int vertex_count;	///< number of vertex points

	int *faces4;		///< faces with 4 vertices
	int faces4_count;	///< number of faces with 4 vertices

	int *faces3;		///< faces with 3 vertices
	int faces3_count;	///< number of faces with 3 vertices

	obj_group_s *groups;	///< structure array with details about group
	int group_count;	///< number of groups

	CObjFile(char *file);
	~CObjFile();
};

#endif
