/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CPRIMITIVES_HPP
#define CPRIMITIVES_HPP

#include <GL/gl.h>
#include <cmath>

class CPrimitives
{
public:
	static void circle(int fans);

	static void round_plate(int fans, float scale = 1.0f);

	static void barrel(int parts, float scale_height = 1.0f, float scale_radius = 1.0f);

	static void plane(int lod = 10, float size=1.0f);

	static void sphere(float r, float cx,float cy,float cz, unsigned int LODi, float texScale = 1);

	static inline void originCross()
	{
		glColor3f(1.0, 0.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(-0.2, 0, 0);
			glVertex3f(1.0, 0, 0);
		glEnd();

		glColor3f(0.0, 1.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(0, -0.2,  0);
			glVertex3f(0, 1.0,  0);
		glEnd();


		glColor3f(0.0, 0.0, 1.0);
		glBegin(GL_LINES);
			glVertex3f(0, 0, -0.2);
			glVertex3f(0, 0, 1.0);
		glEnd();
	};

	/**
	 * very simple arrow
	 */
	static inline void arrow()
	{
		glColor3f(1.0, 0.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(-0.2, 0, 0);
			glVertex3f(1.0, 0, 0);
		glEnd();

		glColor3f(0.0, 1.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(0, -0.2,  0);
			glVertex3f(0, 0.2,  0);
		glEnd();


		glColor3f(0.0, 0.0, 1.0);
		glBegin(GL_LINES);
			glVertex3f(0, 0, -0.2);
			glVertex3f(0, 0, 0.2);
		glEnd();
	};


	static inline void box()
	{
		glBegin(GL_LINE_STRIP);
			glVertex3f(-0.5, -0.5, -0.5);
			glVertex3f(0.5, -0.5, -0.5);
			glVertex3f(0.5, 0.5, -0.5);
			glVertex3f(-0.5, 0.5, -0.5);
			glVertex3f(-0.5, -0.5, -0.5);
			glVertex3f(-0.5, -0.5, 0.5);
			glVertex3f(0.5, -0.5, 0.5);
			glVertex3f(0.5, 0.5, 0.5);
			glVertex3f(-0.5, 0.5, 0.5);
			glVertex3f(-0.5, -0.5, 0.5);
			glEnd();
		glBegin(GL_LINES);
			glVertex3f(0.5, -0.5, -0.5);
			glVertex3f(0.5, -0.5, 0.5);
			glVertex3f(0.5, 0.5, -0.5);
			glVertex3f(0.5, 0.5, 0.5);
			glVertex3f(-0.5, 0.5, -0.5);
			glVertex3f(-0.5, 0.5, 0.5);
		glEnd();
	}
};

#endif
