/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <iostream>
#include <fstream>
#include <list>
#include <limits>
#include <iomanip>
using namespace std;
#include "lib/CVector3.hh"
#include <math.h>

#define MAX_POLYGON_COUNT	28
//#define MC_DEBUG	1


//#define OUTPUT_SHADER
#define OUTPUT_C

/**
 * THIS IS THE REALLY REALLY REALLY UGLY MARCHING CUBES PREGENERATOR
 *
 * AND IT DOES NOT WORK!!!!!! THUS i created the list by myself...
 *
 * dont throw up if you read the code!!!
 *
 * it would be much better to look at the vertice cube as a 45 degree rotated cube...
 */

/**
 * possible vertices in marching cubes
 *
 * position of 0.5 = index/4   !!!
 */
vec3f base_vertices[12] = {
	vec3f(0.5, 0.0, 0.0),	// 0
	vec3f(0.5, 0.0, 1.0),	// 1
	vec3f(0.5, 1.0, 0.0),	// 2
	vec3f(0.5, 1.0, 1.0),	// 3

	vec3f(0.0, 0.5, 0.0),	// 4
	vec3f(1.0, 0.5, 0.0),	// 5
	vec3f(1.0, 0.5, 1.0),	// 6
	vec3f(0.0, 0.5, 1.0),	// 7

	vec3f(1.0, 0.0, 0.5),	// 8
	vec3f(1.0, 1.0, 0.5),	// 9
	vec3f(0.0, 0.0, 0.5),	// 10
	vec3f(0.0, 1.0, 0.5),	// 11
};

/**
 * neighbors to one vertex
 */
int base_vertex_neighbors[12][4];

/**
 * neighbors at side
 * base_vertex_face_neighbors[vertex_nr][side][neighbor_id]
 */
int base_vertex_face_neighbors[12][2][3];


/**
 * return the vertex id within base_vertices
 * \param v	a vertex
 */
int getVertexId(vec3f &v)
{
	for (int i = 0; i < 12; i++)
		if (v == base_vertices[i])
			return i;

	cerr << "vertex not found " << v << endl;
	return -1;
}

/**
 * return the vertex position from 2 neighbored nodes
 */
vec3f getVertexPos(const vec3i &n1, const vec3i &n2)
{
	vec3f vertex;

	vertex = vec3f(
		fabsf(n1.data[0] - n2.data[0])/2.0,
		fabsf(n1.data[1] - n2.data[1])/2.0,
		fabsf(n1.data[2] - n2.data[2])/2.0
	);

	if (n1.data[0] == n2.data[0])
		vertex[0] = n1.data[0];

	if (n1.data[1] == n2.data[1])
		vertex[1] = n1.data[1];

	if (n1.data[2] == n2.data[2])
		vertex[2] = n1.data[2];

	int id = getVertexId(vertex);
	return base_vertices[id];
}

/**
 * return node position of 2 adjacent vertices
 */
vec3i getNodePos(const vec3f &v1, const vec3f &v2)
{
	int half_pos;
	if (v1.data[0] == 0.5)		half_pos = 0;
	else if (v1.data[1] == 0.5)	half_pos = 1;
	else				half_pos = 2;

	vec3f vpos = v1;
	vpos[half_pos] = v2.data[half_pos];

	return vec3i((int)vpos[0], (int)vpos[1], (int)vpos[2]);
}


vec3i getNodePos(int v1, int v2)
{
	return getNodePos(base_vertices[v1], base_vertices[v2]);
}

/**
 * get the vertex id within the base_vertices for 2 neighbored nodes
 */
int getVertexId(const vec3i &n1, const vec3i &n2)
{
	vec3f pos = getVertexPos(n1, n2);
	return getVertexId(pos);
}

/**
 * return compressed node id for vertex
 * \in [0;256[
 */
int getNodeId(vec3i &n)
{
	return (n[2] << 2) + (n[1] << 1) + n[0];
}

/**
 * return node position \in {0,1}[3]
 */
vec3i getNodePos(int p)
{
	return vec3i(
		(p>>0) % 2,
		(p>>1) % 2,
		(p>>2) % 2
	);
}

/**
 * return an id for a normal
 *
 * we just accept normals where one component is set to -1 or 1!
 * the array position of the component plus 1 is multiplied by the component sign and stored
 * e.g. (0,-1,0)
 * => array position of component != 0 is 1
 * the final id becomes (1+1)*-1
 *
 * this normal should just give accurate normal storage
 */
int getNormalId(vec3i v)
{
	if (v[0] != 0)		return 1*v[0];
	else if (v[1] != 0)	return 2*v[1];
	else if (v[2] != 0)	return 3*v[2];

	cerr << "getNormalId INVALID Value" << endl;
	return -1;
}

/**
 * return normal for id
 */
vec3f getNormal(int id)
{
	int aid = abs(id);

	if (aid == 1)		return vec3f(1.0*(id < 0 ? -1.0 : 1.0), 0, 0);
	else if (aid == 2)	return vec3f(0, 1.0*(id < 0 ? -1.0 : 1.0), 0);
	else if (aid == 3)	return vec3f(0, 0, 1.0*(id < 0 ? -1.0 : 1.0));

	cerr << "getNormal INVALID ID" << aid << endl;
	return vec3f();
}


void create_vertexlist(list< pair<int, int> > &new_vertices, int *node_values, list< list< pair<int,int> > > &faces);



// search for nearest vertices
float findNearestVertex(	list< pair<int,int> > &vertices,
				int v0_id,
				list< pair<int,int> >::iterator &ret_iter,
				int v_id_diff
			)
{
	vec3f v0 = base_vertices[v0_id];
	float min_dist = std::numeric_limits<float>::max();

	for (list< pair<int,int> >::iterator i = vertices.begin(); i != vertices.end(); i++)
	{
		if ((*i).first == v_id_diff)	continue;

		int node_id = (*i).first;

		vec3f dv = v0 - base_vertices[node_id];
		float dist = dv[0]*dv[0] + dv[1]*dv[1] + dv[2]*dv[2];
		if (dist < min_dist)
		{
			ret_iter = i;
			min_dist = dist;
		}
	}
	return min_dist;
}


/**
 * search for a vertex in a list and return true if vertex is found (id of vertex)
 * the vertex ist is a pair of the vertex id and the normal id
 * return also the normal
 */
bool isInVertexList(list< pair<int,int> > &l, int v, int *normal = NULL)
{
	for (list< pair<int,int> >::iterator i = l.begin(); i != l.end(); i++)
		if ((*i).first == v)
		{
			if (normal != NULL)
				*normal = (*i).second;
			return true;
		}

	return false;
}



/**
 * output the vertex data
 */
void output_vertices(	int v0_id, int v0_normal_id,
			int v1_id, int v1_normal_id,
			int v2_id, int v2_normal_id
		)
{

#if OUTPUT_SHADER
	cout << setiosflags(ios::fixed) << setprecision(3);
#endif
	// perform a check if the normal aims in the right direction


	vec3f n0 = getNormal(v0_normal_id);
	vec3f n1 = getNormal(v1_normal_id);
	vec3f n2 = getNormal(v2_normal_id);

	// computate average normal of the 3 vertices
	vec3f correct_normal = (n0+n1+n2).getNormal();

	vec3f v0 = base_vertices[v0_id];
	vec3f v1 = base_vertices[v1_id];
	vec3f v2 = base_vertices[v2_id];

#if MC_DEBUG
		cerr << "| " << v0_id << " " << v0_normal_id << "   |";
		cerr << "| " << v1_id << " " << v1_normal_id << "   |";
		cerr << "| " << v2_id << " " << v2_normal_id << "   |" << endl;
		cerr << "| " << v0 << " " << n0 << "   |";
		cerr << "| " << v1 << " " << n1 << "   |";
		cerr << "| " << v2 << " " << n2 << "   |" << endl;
#endif

	// calculate normal
	vec3f d1 = (v1-v0);
	vec3f d2 = (v2-v0);

#if MC_DEBUG
cerr << "correct normal: " << correct_normal << endl;
#endif
	// get perpendicular vector and normalize
	vec3f cross_normal = d1.getCrossProd(d2).getNormal();
	// correct the normal and switch vertices if it's necessary
	if (cross_normal.dot(correct_normal) > 0.0)
	{
		vec3f tmpv = v1;
		v1 = v2;
		v2 = tmpv;
	}
	else
	{
		cross_normal *= -1.0;
	}

	cout << "		{{";
	cout << cross_normal[0] << "," << cross_normal[1] << "," << cross_normal[2];
	cout << "}, {";
	cout << v0[0] << "," << v0[1] << "," << v0[2];
	cout << "}, {";
	cout << v1[0] << "," << v1[1] << "," << v1[2];
	cout << "}, {";
	cout << v2[0] << "," << v2[1] << "," << v2[2];
	cout << "}}," << endl;
/*
	// 2nd check
	vec3f xd0 = v0-v1;
	vec3f xd1 = v0-v2;
	vec3f xnormal = xd0.getCrossProd(xd1).getNormal();
	if (xnormal.dot(correct_normal) > 0.0)
		cerr << "DAMN" << endl;
*/
}


/**
 * PRINT the VERTEX LIST in C FORMAT
 */
void print_output_list(list< pair<int,int> > output_list)
{
	// output as triangles
	while(output_list.size() > 2)
	{
#if MC_DEBUG
//	cerr << "OUTPUT VERTEX" << endl;
#endif
		// VERTEX 1
		int v0_id = output_list.front().first;
		int v0_normal_id = output_list.front().second;
		output_list.pop_front();

		// VERTEX 2
		int v1_id = output_list.front().first;
		int v1_normal_id = output_list.front().second;
		output_list.pop_front();

		// VERTEX 3
		int v2_id = output_list.front().first;
		int v2_normal_id = output_list.front().second;
		output_list.pop_front();

		output_vertices(	v0_id, v0_normal_id,
					v1_id, v1_normal_id,
					v2_id, v2_normal_id
				);
	}
}


/**
 * return valid neighbors to vertex v_id
 * \param vertices	valid vertices
 * \param face_id	the face id where to search neighbors (0 or 1)
 * \param v_id		the vertex where to search neighbors
 * \param neighbors	the neighbor nodes for this vertex and face face_id
 */
void get_vertex_neighbors(list< pair<int,int> > &vertices, int face_id, int v_id, list< pair<int,int> > &neighbors)
{
	// check every neighbored side
	for (int i = 0; i < 3; i++)
	{
		// load possible neighbor
		int nv_id = base_vertex_face_neighbors[v_id][face_id][i];
		int nv_normal_id;

		// check if neighbor exists
		if (isInVertexList(vertices, nv_id, &nv_normal_id))
		{
			/**
			 * we also have to watch that we have not visited that node before
			 * insert to the back so that the first 2 neighbors are the nearest!
			 */
			neighbors.push_back(pair<int,int>(nv_id, nv_normal_id));
		}
	}
}

/**
 * output polygon list given by output_list
 */
void output_polygons(list< pair<int,int> > &output_list)
{
	int polygon_count = output_list.size()/3;

#if MC_DEBUG
cerr << " + OUTPUT POLYGONS: " << polygon_count << " polygons generated" << endl;
#endif
	print_output_list(output_list);

	if (polygon_count > MAX_POLYGON_COUNT)
	{
		cerr << "POLYGONCOUNT: " << polygon_count << endl;
		exit(EXIT_FAILURE);
	}

	if (polygon_count < MAX_POLYGON_COUNT)
	{
		cout << "		";
		for (; polygon_count < MAX_POLYGON_COUNT; polygon_count++)
			cout << "{MN,MN,MN,MN}, ";
		cout << endl;
	}
}


/**
 * here we reconstruct the surface by reconstructing the plane border
 * as the name surface expresses, it should be a kind of "plane" forced to fit through some points.
 * thus it makes sense to find the plane build by outer nodes. this is done by searchine an unique path
 * through the points on the surface
 */
bool create_cube_polygons_from_plane_border(list< pair<int,int> > vertices, list< pair<int,int> > &output_list)
{
#if MC_DEBUG
	cerr << endl;
	cerr << "CREATE CUBE POLYGONS FROM PLANE VERTICES" << endl;
#endif
	list< pair<int,int> > polygon_vertices;

	// walk through all surfaces

#if MC_DEBUG
cerr << "  next check - available vertices: ";
		for (list< pair<int,int> >::iterator iter = vertices.begin(); iter != vertices.end(); iter++)
			cerr << base_vertices[(*iter).first] << "   ";
cerr << endl;
#endif

	// load first vertex (start at random point)
	int v_id = vertices.front().first;
	int v_normal = vertices.front().second;

	int next_v_id = v_id;

	// insert into output list
	polygon_vertices.push_front(pair<int,int>(v_id, v_normal));

	while(vertices.size() != polygon_vertices.size())
	{
		/*
		 * search neighbor to node v_id
		 * there has to be 2 neighbors - each on one side - otherwise we stop and use another method
		 */

		list< pair<int,int> > neighbors0, neighbors1;
		get_vertex_neighbors(vertices, 0, next_v_id, neighbors0);
		get_vertex_neighbors(vertices, 1, next_v_id, neighbors1);

		if (neighbors0.size() != 1 || neighbors1.size() != 1)
		{
#if MC_DEBUG
			cerr << "PLANE FAILED: neighbors0: " << neighbors0.size() << "   neighbors1: " << neighbors1.size() << endl;
			return false;
#endif
		}

		pair<int,int> nv;	// neighbor vertex
		int nv_id;
		int nv_normal_id;
		
		// load neighbor on 1st side
		nv = neighbors0.front();
		nv_id = nv.first;
		nv_normal_id = nv.second;

		if (isInVertexList(polygon_vertices, nv_id, NULL))
		{
			// neighbor on the other side
			nv = neighbors1.front();
			nv_id = nv.first;
			nv_normal_id = nv.second;
			if (isInVertexList(polygon_vertices, nv_id, NULL))
			{
				// no neighbor found? => cannot create unique path
				return false;
			}
		}

		/**
		 * append the list
		 */
		polygon_vertices.push_front(pair<int,int>(nv_id, nv_normal_id));
		next_v_id = nv_id;
	}

	if (polygon_vertices.size() < 3 && polygon_vertices.size() != 0)
	{
		cerr << "POLYGON_VERTICES COUNT < 3" << endl;
		exit(EXIT_FAILURE);
	}

	list< pair<int,int> >::iterator i2 = polygon_vertices.begin();
	list< pair<int,int> >::iterator i0 = i2;
	i2++;
	list< pair<int,int> >::iterator i1 = i2;
	i2++;

	while (i2 != polygon_vertices.end())
	{
		output_list.push_front(*i0);
		output_list.push_front(*i1);
		output_list.push_front(*i2);
//		i0++;
		i1++;
		i2++;
	}

	return true;
}



/**
 * reconstruct main normal according to cube id
 */
int reconstruct_normal(int cube_id, int v0_id, int v1_id, int v2_id)
{
	vec3f v0 = base_vertices[v0_id];
	vec3f v1 = base_vertices[v1_id];
	vec3f v2 = base_vertices[v2_id];

#ifdef MC_DEBUG
	cerr << "reconst normal verts: " << v0 << " " << v1 << " " << v2 << endl;
#endif
	int equal_comp = 0;
	while (v0[equal_comp] != v1[equal_comp] || v1[equal_comp] != v2[equal_comp])
	{
		equal_comp++;
		if (equal_comp == 3)
		{
			cerr << "ERROR: equal_comp == 3" << endl;
			exit(EXIT_FAILURE);
		}
	}

	bool found = false;
	vec3f normal = vec3f(0,0,0);
	for (int b = 0; b < 8; b++)
	{
		if ((1 << b) & cube_id)
		{
			vec3i pos = getNodePos(b);
			if (pos[equal_comp] != (int)v0[equal_comp])
			{
				found = true;
				break;
			}
		}
	}

	int normal_id = equal_comp+1;

	if (v0[equal_comp] == 0)
		normal_id = -normal_id;	// always point away

	if (found)
		normal_id = -normal_id;

	return normal_id;
/*
	if (normal == vec3f(0,0,0))
	{
		cerr << "FUCK" << endl;
		return -(equal_comp+1)*((int)v0[equal_comp]*2-1);
	}

	if (cube_id == 22)
	{
		cerr << normal << endl;
	}

	vec3f absn = vec3f(	fabsf(normal[0]),
				fabsf(normal[1]),
				fabsf(normal[2])
			);

	if (absn[0] > absn[1])
	{	
		if (absn[0] > absn[2])	return (normal[0] < 0 ? 1 : -1);
		else			return (normal[2] < 0 ? 3 : -3);
	}
	else
	{
		if (absn[1] > absn[2])	return (normal[1] < 0 ? 2 : -2);
		else			return (normal[2] < 0 ? 3 : -3);
	}
*/
}



/******************************************************************
 * create CUBE POLYGONS from edge vertices
 * search for vertices with only 2 neighbors and create a polygon
 *
 * \input
 * \faces	the available polygons for a face
 * \polygons	output vertices for GL_TRIANGLES
 */
bool create_cube_polygons_from_edge_vertices(
		list< pair<int,int> > vertices,
		list< pair<int,int> > &output_list,
		int cube_id
)
{
#if MC_DEBUG
	cerr << endl;
	cerr << "CREATE CUBE POLYGONS FROM EDGE VERTICES" << endl;
#endif
	/******************************************************************
	 * iterate through all generated surfaces
	 ******************************************************************/
#if MC_DEBUG
	cerr << " + FACING: " << vertices.size() << endl;
//cerr << "   surface (" << face_count << "): " << face_list.size() << endl;
#endif
	/**
	 * first we handle all those vertices with only 2 or 3 neighbors on one side and no neighbors on the other side and remove them from the list
	 */

	while(vertices.size() > 2)
	{
#if MC_DEBUG
cerr << "  next check - available vertices: ";
		for (list< pair<int,int> >::iterator iter = vertices.begin(); iter != vertices.end(); iter++)
			cerr << base_vertices[(*iter).first] << "   ";
cerr << endl;
#endif
		list< pair<int,int> >::iterator iter;
		for (iter = vertices.begin(); iter != vertices.end(); iter++)
		{
			int v_id = (*iter).first;
			int v_normal_id = (*iter).second;
#if MC_DEBUG
cerr << endl;
cerr << "  checking vertex: " << base_vertices[v_id] << endl;
#endif
			list< pair<int,int> > neighbors0, neighbors1;
			get_vertex_neighbors(vertices, 0, v_id, neighbors0);
			get_vertex_neighbors(vertices, 1, v_id, neighbors1);

#if MC_DEBUG
cerr << "  neighbors0 size: " << neighbors0.size() << endl;
cerr << "  neighbors1 size: " << neighbors1.size() << endl;
#endif
			if (neighbors0.size() == 0)
			{
				// outer edge on face 0 - search for neighbors on face 1

				if (neighbors1.size() == 0 || neighbors1.size() == 1)
				{
#if MC_DEBUG
					cerr << "NEIGHBOR1 SIZE == " << neighbors1.size() << endl;
#endif
					// dont exit in the special case that we want to build up polygons from vertices of [00011000]
					continue;
					exit(EXIT_FAILURE);
				}

				// take first 2 (nearest) neighbors and create polygon

				// load vertices
				int v0_id = v_id;

				list< pair<int,int> >::iterator i = neighbors1.begin();
				int v1_id = (*i).first;

				i++;
				int v2_id = (*i).first;

				// reconstruct normal
				v_normal_id = reconstruct_normal(cube_id, v0_id, v1_id, v2_id);
//				cerr << "reconstructed normal " << v_normal_id << endl;

#if MC_DEBUG
			cerr << base_vertices[v0_id] << "   ";
			cerr << base_vertices[v1_id] << "   ";
			cerr << base_vertices[v2_id] << "   ";
#endif
/*
 * reconstruction is made in main loop
				output_list.push_front(pair<int,int>(v0_id, v_normal_id));
				output_list.push_front(pair<int,int>(v1_id, v_normal_id));
				output_list.push_front(pair<int,int>(v2_id, v_normal_id));
*/
				// remove the edge vertex from vertices
				vertices.erase(iter);
				break;	// continue from the beginning
			}
			else
			{
				if (neighbors1.size() == 0)
				{
					// outer edge on face 1 - search for neighbors on face 0
					if (neighbors0.size() == 0 || neighbors0.size() == 1)
					{
						// there has to be at least 2 neighbors on face 0!
#if MC_DEBUG
						cerr << "NEIGHBOR1 SIZE == " << neighbors1.size() << endl;
#endif
						// dont exit in the special case that we want to build up polygons from vertices of [00011000]
						continue;
						exit(EXIT_FAILURE);
					}

					// take first 2 (nearest) neighbors and create polygon

					// load vertices
					int v0_id = v_id;

					list< pair<int,int> >::iterator i = neighbors0.begin();
					int v1_id = (*i).first;

					i++;
					int v2_id = (*i).first;

					// reconstruct normal
					v_normal_id = reconstruct_normal(cube_id, v0_id, v1_id, v2_id);
//					cerr << "reconstructed normal " << v_normal_id << endl;
#if MC_DEBUG
				cerr << base_vertices[v0_id] << "   ";
				cerr << base_vertices[v1_id] << "   ";
				cerr << base_vertices[v2_id] << "   ";
#endif
/*
 * reconstruction is made in main loop
					output_list.push_front(pair<int,int>(v0_id, v_normal_id));
					output_list.push_front(pair<int,int>(v1_id, v_normal_id));
					output_list.push_front(pair<int,int>(v2_id, v_normal_id));
*/
					// remove the edge vertex from vertices
					vertices.erase(iter);
					break;	// continue from the beginning
				}
				else if (neighbors0.size() == 1 && neighbors1.size() == 1)
				{
					// connect over the edge...
#if MC_DEBUG
				cerr << base_vertices[v_id] << "   ";
#endif
					output_list.push_front(pair<int,int>(v_id, v_normal_id));
					list< pair<int,int> >::iterator i = neighbors0.begin();
#if MC_DEBUG
				cerr << base_vertices[(*i).first] << "   ";
#endif
					output_list.push_front(pair<int,int>((*i).first, (*i).second));
					i = neighbors1.begin();
#if MC_DEBUG
				cerr << base_vertices[(*i).first] << "   " << endl;
#endif
					output_list.push_front(pair<int,int>((*i).first, (*i).second));

					// remove the edge vertex from vertices
					vertices.erase(iter);
					break;	// continue from the beginning
				}
			}
		}// vertices
		if (iter == vertices.end())
			break;
	}

	if (vertices.size() < 3)
		return true;


	if (vertices.size() > 4)
	{
		cerr << "ERROR: vertices.size() > 5" << endl;
		exit(EXIT_FAILURE);
		/*
		if (vertices.size() > 3)
		{
			cerr << "VERTS > 3" << endl;
		}
		*/
	}


	if (vertices.size() == 4)
	{
		/**
		 * 4 vertices are left - that's a special case like case '39'
		 * the first thing we do is to search for an existing edge between 2 of those vertices
		 */
		list< pair<int,int> >::iterator v_iter = vertices.begin();
		int v_id[4];
		int v_neighbor_id[4];
		vec3f v_pos[4];

		for (int i = 0; i < 4; i++)
		{
			v_id[i] = (*v_iter).first;
			v_neighbor_id[i] = (*v_iter).second;
			v_pos[i] = base_vertices[v_id[i]];
			v_iter++;
		}

		// get nearest points to v[0]
		float dist1 = v_pos[0].dist2(v_pos[1]);
		float dist2 = v_pos[0].dist2(v_pos[2]);
		float dist3 = v_pos[0].dist2(v_pos[3]);

		int nv_1, nv_2;
		if (dist1 < dist2)
		{
			// larger distance: dist2
			if (dist3 < dist2)
			{
				// connect to 1 and 3
				nv_1 = 1;
				nv_2 = 3;
			}
			else
			{
				// connect to 1 and 2
				nv_1 = 1;
				nv_2 = 2;
			}
		}
		else
		{
			// larger distance: dist1
			if (dist3 < dist1)
			{
				// connect to 2 and 3
				nv_1 = 2;
				nv_2 = 3;
			}
			else
			{
				// connect to 2 and 1
				nv_1 = 2;
				nv_2 = 1;
			}
		}

		output_list.push_front(pair<int,int>(v_id[0], v_neighbor_id[0]));
		output_list.push_front(pair<int,int>(v_id[nv_1], v_neighbor_id[nv_1]));
		output_list.push_front(pair<int,int>(v_id[nv_2], v_neighbor_id[nv_2]));

		vertices.pop_front();
		// let the rest be done by the next vertices.size()==3 check
	}

	if (vertices.size() == 3)
	{
		/**
		 * there are 3 vertices left -> connect them directly
		 */
		list< pair<int,int> >::iterator i = vertices.begin();
		// load vertices
		int v0_id = (*i).first;
		int v0_normal_id = (*i).second;
		i++;
		int v1_id = (*i).first;
		int v1_normal_id = (*i).second;
		i++;
		int v2_id = (*i).first;
		int v2_normal_id = (*i).second;

#if MC_DEBUG
	cerr << base_vertices[v0_id] << "   ";
	cerr << base_vertices[v1_id] << "   ";
	cerr << base_vertices[v2_id] << "   ";
#endif
		output_list.push_front(pair<int,int>(v0_id, v0_normal_id));
		output_list.push_front(pair<int,int>(v1_id, v1_normal_id));
		output_list.push_front(pair<int,int>(v2_id, v2_normal_id));

		return true;
	}
	return true;
}




/******************************************************************
 * MAIN FUNCTION
 */
int main(int argc, char** argv)
{
	// setup base_vertex_neighbors
	for (int i = 0; i < 12; i++)
	{
		vec3f &v = base_vertices[i];

		int half_pos;
		if (v[0] == 0.5)	half_pos = 0;
		else if (v[1] == 0.5)	half_pos = 1;
		else			half_pos = 2;

		vec3f n;

	// VERTEX NEIGHBORS
		n = v;
		n[half_pos] = 0.0;
		n[(half_pos+1)%3] = 0.5;
		base_vertex_neighbors[i][0] = getVertexId(n);
		// setup normal - use half pos as direction

		n[half_pos] = 1.0;
		base_vertex_neighbors[i][1] = getVertexId(n);

		n = v;
		n[half_pos] = 0.0;
		n[(half_pos+2)%3] = 0.5;
		base_vertex_neighbors[i][2] = getVertexId(n);

		n[half_pos] = 1.0;
		base_vertex_neighbors[i][3] = getVertexId(n);

	// FACE NEIGHBORS
	// they have to be stored in an special format
	// 1st place: one of the nearest nodes
	// 2nd plance: the other of the nearest nodes
	// 3rd place: one of the most distance nodes
		int var;	// 2nd variable component

	// 1st surface
		var = (half_pos+1)%3;

		// direct neighbored vertex
		n = v;
		n[var] = 0.5;
		n[half_pos] = 0.0;
		base_vertex_face_neighbors[i][0][0] = getVertexId(n);

		// other direct neighbored vertex
		n = v;
		n[var] = 0.5;
		n[half_pos] = 1.0;
		base_vertex_face_neighbors[i][0][1] = getVertexId(n);

		// opposite vertex
		n = v;
		if (n[var] == 1.0)	n[var] = 0.0;
		else			n[var] = 1.0;
		base_vertex_face_neighbors[i][0][2] = getVertexId(n);


	// 2nd surface
		var = (half_pos+2)%3;

		// direct neighbored vertex
		n = v;
		n[var] = 0.5;
		n[half_pos] = 0.0;
		base_vertex_face_neighbors[i][1][0] = getVertexId(n);

		// other vertex
		n = v;
		n[var] = 0.5;
		n[half_pos] = 1.0;
		base_vertex_face_neighbors[i][1][1] = getVertexId(n);

		// opposite vertex
		n = v;
		if (n[var] == 1.0)	n[var] = 0.0;
		else			n[var] = 1.0;
		base_vertex_face_neighbors[i][1][2] = getVertexId(n);

#if MC_DEBUG
		cerr << v << " ";
		cerr << base_vertices[base_vertex_face_neighbors[i][0][0]] << " ";
		cerr << base_vertices[base_vertex_face_neighbors[i][0][1]] << " ";
		cerr << base_vertices[base_vertex_face_neighbors[i][0][2]] << " ";
		cerr << base_vertices[base_vertex_face_neighbors[i][1][0]] << " ";
		cerr << base_vertices[base_vertex_face_neighbors[i][1][1]] << " ";
		cerr << base_vertices[base_vertex_face_neighbors[i][1][2]] << " ";
		cerr << endl;
#endif
/*
		cerr << base_vertex_face_neighbors[i][0][0] << " ";
		cerr << base_vertex_face_neighbors[i][0][1] << " ";
		cerr << base_vertex_face_neighbors[i][0][2] << " ";
		cerr << base_vertex_face_neighbors[i][1][0] << " ";
		cerr << base_vertex_face_neighbors[i][1][1] << " ";
		cerr << base_vertex_face_neighbors[i][1][2] << " ";
*/
//		cerr << endl;

	}
//	exit(1);


	/**
	 * maximums:
	 *
	 * 4 surfaces per cube (one corner gains exclusive rights to the neighbors)
	 * polygons per surface: 4 (6 nodes)
	 */
	cout << "#define MN	{-111.0,-111.0,-111.0}" << endl;
	cout << "#define MC_MAX_POLYGON_COUNT " << MAX_POLYGON_COUNT << endl;
#if OUTPUT_SHADER
	cout << "vec3 marching_cubes[256][" << MAX_POLYGON_COUNT << "][4] = {" << endl;
#else
	cout << "// storage: normal, vertex, vertex, vertex" << endl;
	cout << "float marching_cubes[256][" << MAX_POLYGON_COUNT << "][4][3] = {" << endl;
#endif

	/******************************************************************
	 * walk through possible nodes
	 ******************************************************************/
	for (int p = 0; p < 256; p++)
//	for (int p = 0; p < 50; p++)
//	int p = 39;
	{
#if MC_DEBUG
		cerr << p << endl;
#endif
		/*
		 * prepare flags for every node
		 * the flags are decoded from the enumeration of states as a bitmask
		 */
		int node_values[8];
		for (int i = 0; i < 8; i++)
			node_values[i] = 1 & (p >> i);

		list< pair<int,int> > new_vertices;

		/*
		 * search for inner neighbors (value == 1)
		 */
		for (int n1 = 0; n1 < 8; n1++)
		{
			if (node_values[n1])	// dont start searching from inner surface (value == 1)
				continue;

			vec3i node1 = getNodePos(n1);		// get position of node
			int node_value = node_values[n1];	// value of this node (in or out of surface)

			vec3i node_neighbor[3];		// setup neighbors to node
			node_neighbor[0] = vec3i(!(node1[0]), node1[1], node1[2]);
			node_neighbor[1] = vec3i(node1[0], !(node1[1]), node1[2]);
			node_neighbor[2] = vec3i(node1[0], node1[1], !(node1[2]));

			for (int c = 0; c < 3; c++)
			{
				vec3i &node2 = node_neighbor[c];	// neighbor position
				int n2 = getNodeId(node2);		// neighbor node id

				if (node_values[n2] != node_value)	// valid node value (opposite of neighbor searching node)
				{
					int vertex_id = getVertexId(node1, node2);	// get vertex id (vertex between both nodes)
					int normal_id = getNormalId(node1-node2);	// get normal id from vector
					new_vertices.push_back(pair<int,int>(vertex_id, normal_id));	// store vertex id
				}
			}
		}

		// output the stuff

		cout << "{	// cube " << p << ": ";

		int *t = node_values;
		cout << "[" << t[0] << t[1] << t[2] << t[3] << t[4] << t[5] << t[6] << t[7] << "] ";

#if MC_DEBUG
	cerr << "[" << t[0] << t[1] << t[2] << t[3] << t[4] << t[5] << t[6] << t[7] << "] " << endl;
#endif
		cout << endl;

		/**
		 * create the vertex list
		 */
		list< list< pair<int,int> > > faces;
		create_vertexlist(new_vertices, node_values, faces);

#if MC_DEBUG
		cerr << "  surfaces: " << faces.size() << endl;
#endif

		/**
		 * create the cube polygons
		 */
		list< pair<int,int> > output_list;
		for (list< list< pair<int,int> > >::iterator iter = faces.begin(); iter != faces.end(); iter++)
		{
			list< pair<int,int> > vertices = *iter;
			list< pair<int,int> > tmp_output_list;

#if MC_DEBUG
			cerr << endl;
			cerr << "CREATING CUBE POLYGONS" << endl;
#endif
			// 1st try
			// use this as first try to create appropriate triangles at the corner
			if (!create_cube_polygons_from_edge_vertices(vertices, tmp_output_list, p))
			{
				cerr << "FAILED TO CREATE MC's" << endl;
				exit(EXIT_FAILURE);
			}
#if MC_DEBUG
			cerr << "NEW VERTICES: " << tmp_output_list.size() << endl;
#endif
			for (list< pair<int,int> >::iterator i = tmp_output_list.begin(); i != tmp_output_list.end(); i++)
				output_list.push_back(*i);
		}

		/**
		 * now we should have a look at the surfaces if it's necessary to restore plates on those
		 * this is necessary if e.g. this construction exists:
		 * ---X---
		 * |     |
		 * 0     O
		 * |     |
		 * ---X---
		 */

		for (int axis = 0; axis < 3; axis++)
		{
			int comp0 = (axis+0)%3;
			int comp1 = (axis+1)%3;
			int comp2 = (axis+2)%3;

			vec3i pos;
			for (int comp_val = 0; comp_val < 2; comp_val++)
			{
				pos[comp0] = comp_val;	// 0 for 1st component

				pos[comp1] = 0;	pos[comp2] = 0;
				vec3i v00_pos = pos;
				int v00_id = getNodeId(pos);
				int value00 = node_values[v00_id];

				pos[comp1] = 0;	pos[comp2] = 1;
				vec3i v01_pos = pos;
				int v01_id = getNodeId(pos);
				int value01 = node_values[v01_id];

				pos[comp1] = 1;	pos[comp2] = 0;
				vec3i v10_pos = pos;
				int v10_id = getNodeId(pos);
				int value10 = node_values[v10_id];

				pos[comp1] = 1;	pos[comp2] = 1;
				vec3i v11_pos = pos;
				int v11_id = getNodeId(pos);
				int value11 = node_values[v11_id];

				if (	(value00 == value11) &&
					(value01 == value10) &&
					(value00 != value01)
				)
				{
					// draw rectangle
					int v0_id = getVertexId(v00_pos, v10_pos);	vec3f v0 = base_vertices[v0_id];
					int v1_id = getVertexId(v10_pos, v11_pos);	vec3f v1 = base_vertices[v1_id];
					int v2_id = getVertexId(v11_pos, v01_pos);	vec3f v2 = base_vertices[v2_id];
					int v3_id = getVertexId(v01_pos, v00_pos);	vec3f v3 = base_vertices[v3_id];

					int normal_id = reconstruct_normal(p, v0_id, v1_id, v2_id);

					output_list.push_front(pair<int,int>(v0_id, -normal_id));
					output_list.push_front(pair<int,int>(v1_id, -normal_id));
					output_list.push_front(pair<int,int>(v2_id, -normal_id));

					output_list.push_front(pair<int,int>(v0_id, -normal_id));
					output_list.push_front(pair<int,int>(v2_id, -normal_id));
					output_list.push_front(pair<int,int>(v3_id, -normal_id));
				}
			}
		}

		output_polygons(output_list);

		cout << "},	// cube" << endl;
	}
	cout << "};	// marching cubes" << endl;

	return EXIT_SUCCESS;
}




/**
 * delete a vertex from a list
 */
void deleteFromVertexList(list< pair<int,int> > &l, int &v)
{
	for (list< pair<int,int> >::iterator i = l.begin(); i != l.end(); i++)
		if ((*i).first == v)
		{
			l.erase(i);
			return;
		}
	cerr << "DELETE: vertex not found " << v << endl;
}


bool isInList(list<vec3f> &l, vec3f &v)
{
	for (list<vec3f>::iterator i = l.begin(); i != l.end(); i++)
		if (*i == v)
			return true;

	return false;
}

void deleteFromList(list<vec3f> &l, vec3f &v)
{
	for (list<vec3f>::iterator i = l.begin(); i != l.end(); i++)
		if (*i == v)
		{
			l.erase(i);
			return;
		}
	cerr << "DELETE: vertex not found " << v << endl;
}



/**
 * continue searching vertex neighbors
 */
void search_side_neighbors(	list< pair<int,int> > &new_vertices,	// list with available vertices
				list< pair<int,int> > &face_list,		// output list to generate face
				bool *vertices_visited,		// array with visited neighbors \in [0;12[
				int *node_values,		// values for nodes (1: inside, 0: outside)
				int vertex_id,			// vertex id of recent visited vertex
				int gate_value			// the value to cross neighbored nodes
			)
{
#if MC_DEBUG
	cerr << "checking neighbors for node " << base_vertices[vertex_id] << endl;
#endif
	// check all neighbors
	for (int i = 0; i < 4; i++)
	{
		// new vertex id of neighbor
		int nv_id = base_vertex_neighbors[vertex_id][i];
#if MC_DEBUG
		cerr << "    " << vertex_id << ": checking neighbor: " << nv_id << "       " << base_vertices[nv_id] << endl;
#endif
		// if neighbor is not already visited...
		if (!vertices_visited[nv_id])
		{
//			cout << " walk on (" << base_vertices[vertex_id] << " -> " << base_vertices[nv_id] << ")....";

			// ...set to visited
			vec3i node_pos = getNodePos(vertex_id, nv_id);
			int n_node_id = getNodeId(node_pos);
#if MC_DEBUG
	cerr << " over neighbor node " << n_node_id << "(" << node_pos << ")" << endl;
#endif
			if (node_values[n_node_id] == gate_value)	// only walk over nodes with value == gate_value
			{
				vertices_visited[nv_id] = true;
				// recalculate node which was crossed by walkint this way
				int normal_id;
				if (isInVertexList(new_vertices, nv_id, &normal_id))
				{
//					if (gate_value == 1)
//						normal_id = -normal_id;
#if MC_DEBUG
					cerr << "--- INSERT into face list: " << base_vertices[nv_id] << endl;
#endif
					face_list.push_front( pair<int,int>(nv_id, normal_id) );
					deleteFromVertexList(new_vertices, nv_id);
				}
				else
				{
#if MC_DEBUG
//					cerr << "NOT in vertex list" << endl;
#endif
				}
				search_side_neighbors(new_vertices, face_list, vertices_visited, node_values, nv_id, gate_value);
			}
		}
	}
}



/**
 * \input
 * 	new_vertices:	a list with vertices
 * 	node_values:	1 or 0 respective if the node is out or in relative to the iso surface
 *
 * \output:	cout a triangles to render the surface
 *
 * we use the property that the normal of the created polygons always have to aim
 * wether to an out or in node. so we have to walk over out or in laying nodes to connect
 * to the neighbored nodes
 */
void create_vertexlist(list< pair<int, int> > &new_vertices, int *node_values, list< list< pair<int,int> > > &faces)
{
	if (new_vertices.size() == 0)
		return;

	if (new_vertices.size() < 3)
	{
		cerr << "ERROR: new vertices for face < 3!" << endl;
		return;
	}

	int gate_value = 0;
	list< pair<int, int> > copy_vertices = new_vertices;	// backup vertices
	list< list< pair<int, int> > > first_faces;
	int previous_faces = -1;

again:
	// we store neighbored nodes in this list
	faces.clear();
	while(new_vertices.size() > 0)
	{
		faces.push_front(list< pair<int,int> >());
		list< pair<int,int> > &face_list = faces.front();

		// take next vertex
		int v = new_vertices.front().first;
		int n = new_vertices.front().second;
		new_vertices.pop_front();	// and remove it
		face_list.push_front(pair<int,int>(v, n));

		// array with vertex flags to determine if vertex was already visited
		bool vertex_visited[12];
		for (int i = 0; i < 12; i++)
			vertex_visited[i] = false;

		search_side_neighbors(new_vertices, face_list, vertex_visited, node_values, v, gate_value);

#if MC_DEBUG
		cerr << "NEW FACE: " << face_list.size() << " vertices" << endl;
#endif
	}


	int max = std::numeric_limits<int>::min();
	for (list< list< pair<int,int> > >::iterator i = faces.begin(); i != faces.end(); i++)
		if ((int)(*i).size() > max)
			max = (*i).size();

	if (previous_faces == -1)
	{
#if MC_DEBUG
		cerr << "first max face size: " << max << endl;
#endif
		previous_faces = max;
//		previous_faces = faces.front().size();
		if (previous_faces > 6)
		{
#if MC_DEBUG
			cerr << "recreating face (too long)..." << faces.front().size() << endl;
#endif
			first_faces = faces;
//			first_vertices = new_vertices;	// backup vertices
//	cerr << "first: " << first_vertices.size() << endl;
			gate_value = 1;			// set opposite gate value
			new_vertices = copy_vertices;	// restore old vertices
			goto again;
		}
	}
	else
	{
#if MC_DEBUG
		cerr << "second max face size: " << max << endl;
#endif
//		if (previous_faces < faces.front().size())
		if (previous_faces < max)
		{
//	cerr << "first: " << first_vertices.size();
#if MC_DEBUG
			cerr << "taking first vertices" << endl;
#endif
			// if the previous try was really better, we restore those of the first round
			faces = first_faces;
//			new_vertices = first_vertices;
		}
	}

}
