/*
 * This file is part of the Lattice Boltzmann Open Surface program (lbm_os).
 *
 * lbm_os is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lbm_os is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lbm_os.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <iostream>
using namespace std;
#include "CProgram.hpp"


/**
 * print information how to use the program
 */
void printUsage(string &prog)
{
	cerr << endl;
	cerr << "usage: " << prog << " [shader_file(.vert|.frag)] [-e]" << endl;
	cerr << "	-e: enumerate lines" << endl;
	cerr << endl;
}

int main(int argc, char **argv)
{
	bool enumerateLines = false;
	string prog = argv[0];
	if (argc < 2 || argc > 3)
	{
		printUsage(prog);
		exit(EXIT_FAILURE);
	}

	if (argc == 3)
	{
		if (string("-e").compare(argv[2]) != 0)
		{
			printUsage(prog);
			exit(EXIT_FAILURE);
		}
		enumerateLines = true;
	}

	CProgram program(false);

	if (program.error)
	{
		cerr << "error: " << program.errorLog << endl;
		exit(EXIT_FAILURE);
	}

	string fileName = argv[1];
	string code;

	program.loadAndPrecompile(argv[1], code);

	if (program.error)
	{
		cerr << "error: " << program.errorLog << endl;
		exit(EXIT_FAILURE);
	}

	if (argc == 3)
	{
		SShaderCode scode;
		scode.code = code;
		scode.output(false);
	}
	else
	{
		cout << code << endl;
	}

	return 0;
}
